//---------------------------------------------------------------------------
#ifndef CommonH
#define CommonH
//---------------------------------------------------------------------------
#include "Threads.h"

#include <jpeg.hpp>
#include <dir.h>
#include <complex>
using namespace std;
#define M_PI 3.14159265358979323846
#define PRECISION long double

#define EYECAT "Mandelbrot Root Explorer"
#define SOFTWARE_VERSION   1
#define SOFTWARE_REVISION  4
#define PARAMETER_VERSION  1
#define PARAMETER_REVISION 0

#define MAX_DIMENSION 384       // must be 8*n
#define REV_MAX_DIMENSION 16384 // must be 8*n
#define DEFAULT_MAXITER 512
#define MAXITER_PER_DIMENSION 80
#define MAXITER_PER_RETRY 128

#define ROOT_IMAGE_SIZE_X 300  // can't change
#define ROOT_IMAGE_SIZE_Y 225  // can't change
#define DEFAULT_ROOT_IMAGE_CENTER_X -0.5
#define DEFAULT_ROOT_IMAGE_CENTER_Y 0.0
#define DEFAULT_ROOT_IMAGE_SPAN_X 4.0
#define DEFAULT_ROOT_IMAGE_SPAN_Y 3.0
#define ROOT_FIND_TRIAL_ZOOM_POWER 5.0
#define ROOT_FIND_TRIAL_DIM_PER_RETRY 64
#define ROOT_NUMBER_MAX 8
#define NEWTON_EPSILON LDBL_EPSILON*10000
#define NEWTON_REPEAT 16
#define MANDELBROT_SPAN_RATIO 0.03
#define MANDELBROT_SPAN_LIMIT LDBL_EPSILON*10000
#define MANDELBROT_SPAN_LIMIT2 MANDELBROT_SPAN_LIMIT/100

#define IMAGE_SIZE_MAX 2048  // must be 8*n
#define IMAGE_SIZE_MIN 50
#define DEFAULT_IMAGE_SIZE_X 800
#define DEFAULT_IMAGE_SIZE_Y 600
#define DEFAULT_IMAGE_CENTER_X -0.5
#define DEFAULT_IMAGE_CENTER_Y 0.0
#define DEFAULT_IMAGE_SPAN_X 4.0
#define DEFAULT_IMAGE_SPAN_Y 3.0
#define MALTI_PASS_NUMBER 4
#define POWER_FOR_PASS 2     // 32pix, 16pix, 4pix, 1pix

#define COLORS 256  // can't change
#define PAINT_COUNT 3200

#define DIR_DEMO "Demo\\"
#define DIR_OPEN "Save\\"
#define DIR_SAVE "Save\\"
#define DIR_SAVEAS "Save\\"
#define DIR_MAP "Map\\"
#define DIR_MOVIE "Movie\\"
#define EXT_DEMO ".demo"
#define EXT_PAR ".par"
#define EXT_BMP ".bmp"
#define EXT_JPG ".jpg"
#define EXT_MAP ".map"
#define EXT_SPR ".spr"
#define EXT_ZMOV ".zmov"
#define EXT_SMOV ".smov"
#define FILE_ROOT_MAP "RootDefault"
#define FILE_FRACTAL_MAP "FractalDefault"
#define INITIAL_FILE "MandelbrotRootExplorer.init"

enum FORMULA { Mandelbrot, Julia };
enum IMAGE_XY { X, Y };
enum IMAGE_EXT { Bmp, Jpg };
enum DRAWING { Non, Root, Fractal, ColorCycling, SlideShow, ReverseShow, SpiralShow };

typedef struct {
  TLogPalette   pal;
  TPaletteEntry entry[COLORS - 1];
} LogPalette;

typedef struct
{
    char               eyecat[64];
    int                parameterVersion;
    short              parameterRevision;
    char               insideOption;
    char               outsideOption;
    FORMULA            formula;
    int                dimension;
    unsigned char      nthRoot[MAX_DIMENSION/8];
    int                periodicBitsNumber;
    unsigned char      periodicBitsValue[MAX_DIMENSION/8];
    int                topBitsNumber;
    unsigned char      topBitsValue[MAX_DIMENSION/8];
    int                lastBitsNumber;
    unsigned char      lastBitsValue[MAX_DIMENSION/8];
    int                nthRootAtRandom;
    int                nthRootAtRandomKind;
    int                nthRootModifyBits;
    complex<PRECISION> pCenter;    // root X,Y
    PRECISION          pSpan[Y+1];
    complex<PRECISION> c0Center;   // zero
    PRECISION          c0Span[Y+1];
    int                maxIter;
    PRECISION          rotationAngle;
    int                imageSize[Y+1];
    LogPalette         lPal;
} Parameter;

typedef struct {
    char path[MAXPATH];
    char drive[MAXDRIVE];
    char dir[MAXDIR];
    char file[MAXFILE];
    char ext[MAXEXT];
} FilePath;

typedef struct {
    PRECISION          norm;
    complex<PRECISION> p;
} NewtonStart;

typedef struct {
    char      startPath[MAXPATH];
    FilePath  demoPath;
    FilePath  openPath;
    FilePath  savePath;
    FilePath  saveAsPath;
    FilePath  mapPath;
    FilePath  moviePath;

    int       screenWidth;
    Graphics::TBitmap *fractalBitmap;
    Graphics::TBitmap *rootBitmap;
    Graphics::TBitmap *colorBitmap;
    Graphics::TBitmap *rgbBitmap[3];

    int       fractalChanged;
    int       centerChanged;
    int       spanChanged;
    int       rootChanged;
    int       rootFindNonStop;
    int       rootFindHiddenArea;
    int       rootFindWithFractal;
    PRECISION rootFindZoomInPower;
    PRECISION rootFindNowZoomInPower;
    int       rootFindRetryMaxAuto;
    int       rootFindRetryMax;
    int       rootFindRetryN;
    complex<PRECISION> rootFindCenterP;
    complex<PRECISION> rootPCenter;
    PRECISION rootPSpan[Y+1];
    int       hiddenDimension[ROOT_IMAGE_SIZE_Y][ROOT_IMAGE_SIZE_X];
    PRECISION hiddenReal[ROOT_IMAGE_SIZE_Y][ROOT_IMAGE_SIZE_X];
    NewtonStart newtonStartX[ROOT_IMAGE_SIZE_X];
    NewtonStart newtonStartY[ROOT_IMAGE_SIZE_Y];
    int       rootNumber;
    complex<PRECISION> rootP[ROOT_NUMBER_MAX];
    int       rootWhich;
    int       firstImage;
    int       passMulti;
    int       passMultiNow;
    int       pass;
    int       passPixels;
    int       prevPassPixels;
    unsigned char diverged[IMAGE_SIZE_MAX][IMAGE_SIZE_MAX/8];
    int       fractalCalculationCompleted;

    DRAWING   onDrawing;
    int       threadTempStop;
    int       threadTerminate;
    int       onTempStop;

    TThreads  *threads;
    int       rootBufToggle;
    int       rootCalculatedY;
    int       rootShowed[2];
    BYTE      rootBuf[2][ROOT_IMAGE_SIZE_X];
    int       fractalBufToggle;
    int       fractalCalculatedY;
    int       fractalCalculatedX;
    int       fractalBufCalculatedN[2];
    BYTE      fractalBuf[2][PAINT_COUNT];

    int       zoomMode;
    int       onZoom;
    PRECISION zoomMag;
    int       moveCenter;
    int       zoomCenterX;
    int       zoomCenterY;
    int       clickIgnore;
    int       returnToMand;
    PRECISION returnToMandPSpan[Y+1];
    int       formSizeAuto;

    complex<PRECISION> center;
    PRECISION span[Y+1];
    int       maxIterAuto;
    int       saveImage;
    IMAGE_EXT saveImageExt;
    int       rootNonStopFound;
    complex<PRECISION> rootNonStopCenterP;
    int       zoomInPowerOnEdit;
    int       maxRetryOnEdit;

    int       reverseAnalysis;
    int       reverseAnalysisZoom;
    int       savedZoomMode;
    int       reverseShowMode;
    complex<PRECISION> revRoot1;
    complex<PRECISION> revRoot2;
    complex<PRECISION> boxCenter;
    int       reverseShowX0;
    int       reverseShowY0;
    int       reverseShowX1;
    int       reverseShowY1;
    int       reverseShowX2;
    int       reverseShowY2;
    int       headNotFound;
    int       save1Dimension;
    int       save2Dimension;
    unsigned char save1NthRoot[REV_MAX_DIMENSION/8];
    unsigned char save2NthRoot[REV_MAX_DIMENSION/8];
    int       dotLine;
    int       topNext;
    int       leftBit;
    int       insBit;
    int       rightBit;
    int       insCount;
    unsigned char leftNthRoot[REV_MAX_DIMENSION/8];
    unsigned char insNthRoot[REV_MAX_DIMENSION/8];
    unsigned char rightNthRoot[REV_MAX_DIMENSION/8];
    int       spiralShow;
    int       spiralStopReason;
    int       spiralTimes;
    int       spiralDimension;
    unsigned char spiralNthRoot[REV_MAX_DIMENSION/8];
    unsigned char curInsNthRoot[REV_MAX_DIMENSION/8];
    complex<PRECISION> topRoot;
    complex<PRECISION> nextRoot;
    complex<PRECISION> spiralRoot;
    int       spiralBitReversed;
    int       spiralBitReversedLeft;
    int       spiralBitReversedIns;
    int       spiralBitReversedInsed;
    int       spiralBitReversedRight;
    int       spiralRetryMesh;
    int       spiralRetryMeshMax;
    int       spiralRetryBitRev;
    int       spiralRetryBitRevMax;

    int       changeRGBIndex;
    int       cyclingReverse;
    int       cyclingAuto;
    int       cyclingTimes;
    LogPalette rootLPal;
    int       editRGB;
    int       leftAnchor;
    int       rightAnchor;
    int       changePoint;
    int       leftColor[3];
    int       rightColor[3];
    int       changeColor[3];
    int       leftGradient[3];
    int       rightGradient[3];
    int       verticalScrollOnAjust;

    int       movImageOpened;
    FILE      *movListFp;
    char      movListFileName[MAXFILE];
    int       movieReverse;
    int       movieDelay;
    int       moviePainted;
    IMAGE_EXT movieExt;
    int       movieSkipNoImage;
} MakeImage;

typedef struct {
    char      demoPath[MAXPATH];
    int       fitToScreen;
    int       sizeDefault[Y+1];
    int       zoomAutoChange;
    int       zoomModeDefault;
    int       passMultiDefault;
} Initial;

typedef struct {
    int       onDemo;
    int       nonParFile;
    FILE      *fp;
    int       infinite;
    int       findingRoot;
    int       showRoot;
    char      caption[256];
    int       maxRetry;       // -1 : use default
    int       imageSize[Y+1]; // -1 : use size in par
    int       spiralTimes;
    FilePath  parPath;
    FilePath  sprPath;
} DemoInf;

typedef struct {
    complex<PRECISION> topBoxCenter;
    PRECISION topBoxSpanX;
    PRECISION topBoxSpanY;
    complex<PRECISION> nextBoxCenter;
    PRECISION nextBoxSpanX;
    PRECISION nextBoxSpanY;
    int       dotLine;
} SpiralInf;

typedef struct {
    int       onDemo;
    int       fromDimension;
    int       toDimension;
    int       dimension;
    int       nthRoot;
} LowDimInf;

typedef struct {
    int       onMakingMovie;
    int       zoomOrSim;
    int       par0Set;
    int       par1Set;
    Parameter par0;
    Parameter par1;
    int       nowSerialNumber;
    int       numberOfImages;
    PRECISION zoomInPower;
} MovieInf;

typedef struct {
    complex<PRECISION> center;
    PRECISION span[Y+1];
    complex<PRECISION> constant;
    int       maxIter;
    PRECISION rotationAngle;
} MovieList;

#ifdef MAIN_FRACTAL_ROUTINE
    Parameter par;
    MakeImage img;
    Initial   init;
    Initial   initEdit;
    DemoInf   demo;
    SpiralInf spr;
    LowDimInf low;
    MovieInf  mov;
    MovieList list;
#else
    extern Parameter par;
    extern MakeImage img;
    extern Initial   init;
    extern Initial   initEdit;
    extern DemoInf   demo;
    extern SpiralInf spr;
    extern LowDimInf low;
    extern MovieInf  mov;
    extern MovieList list;
#endif

/* in Initialize.cpp */
void fractalInitialize();
void resetParameters();
void resetAndShow();
void setDemoPath(char *path, char *file, char *ext);
void setOpenPath(char *path, char *file, char *ext);
void setSavePath(char *path, char *file, char *ext);
void setSaveAsPath(char *path, char *file, char *ext);
void setMapPath(char *path, char *file, char *ext);
void setMoviePath(char *path, char *file, char *ext);
void setPath(char *path, char *file, char *ext, FilePath *filePath);
bool openPar(char *path, int clearAtRandom);
bool loadImage();
void savePar(int saveMode);
void saveImage(int saveMode);
void makeFileName(char *file);
bool openSpiral(char *path);
void saveSpiral();
/* in Root.cpp */
void controlFormInitialize();
void rootImageCalculationStart();
void setRandomNthRoot();
void rootImageCalculateInThread();
void rootImageShow(int bufToggle);
void rootImageCalculationEnd();
bool getRootByNewtonMethod(int dimension, unsigned char *nthRoot,
         complex<PRECISION> *newtonStartP, complex<PRECISION> *root);
void showRootPosition(int colorRed);
void showCenterPosition(complex<PRECISION> center);
void showFindPosition();
void reverseAnalysisFormInitialize();
void reverseAnalysis(complex<PRECISION> boxCenter, PRECISION boxSpanX, PRECISION boxSpanY);
int  reverseCalculation(complex<PRECISION> boxCenter, PRECISION boxSpanX, PRECISION boxSpanY,
         int *revDimension, unsigned char *revNthRoot, complex<PRECISION> *root);
void reverseShowInThread();
void reverseShow();
void bodyAndHeadFormInitialize();
void bodyAndHead(complex<PRECISION> boxCenter, PRECISION boxSpanX, PRECISION boxSpanY);
int  getHeadRoot(complex<PRECISION> boxCenter, PRECISION boxSpanX, PRECISION boxSpanY,
         int bodyDimension, unsigned char *bodyNthRoot,
         int *midBit, unsigned char *headNthRoot, complex<PRECISION> *headRoot);
void splitHeadNthRoot(int bodyDimension, unsigned char *bodyNthRoot, unsigned char *headNthRoot,
         int *leftReversed, int *rightReversed,
         int *midBit, unsigned char *bodyNthRootLeft, unsigned char *bodyNthRootRight);
bool getRootInBox(complex<PRECISION> boxCenter, PRECISION boxSpanX, PRECISION boxSpanY,
         int mesh, PRECISION allowance, int dimension, unsigned char *nthRoot, complex<PRECISION> *root);
void spiralFormInitialize();
void spiral(complex<PRECISION> boxCenter, PRECISION boxSpanX, PRECISION boxSpanY);
int  splitSpiralNthRoot(int topDimension, unsigned char *topNthRoot,
         int nextDimension, unsigned char *nextNthRoot,
         int *leftBit, int *insBit, int *rightBit, int *insCount,
         unsigned char *leftNthRoot, unsigned char *insNthRoot, unsigned char *rightNthRoot);
void spiralDrawRect(complex<PRECISION> boxCenter, PRECISION boxSpanX, PRECISION boxSpanY);
void spiralGo();
void spiralShowInThread();
int  mergeSpiralNthRoot(int leftBit, int insedBit, int insBit, int rightBit,
         unsigned char *leftNthRoot, unsigned char *insedNthRoot,
         unsigned char *insNthRoot, unsigned char *rightNthRoot,
         int *dimension, unsigned char *nthRoot);
void spiralShow(int dotLine);
void spiralRetry();
void spiralStop();
void spiralEndMessage();
void changeToBoxMode();
void restoreZoomMode();
void startLow();
/* in Fractal.cpp */
void imageFormInitialize(int clear);
void fractalCalculationStart();
void defaultSpan(PRECISION *defSpan);
void estimateMandelbrotSpan();
void fractalCalculationNextPass();
void fractalCalculateInThread();
int  fractalIteration(int maxIter, PRECISION pReal, PRECISION pImag,
         PRECISION *cReal, PRECISION *cImag);
int  fractalIterationWithDistance(int maxIter, PRECISION pReal, PRECISION pImag,
         PRECISION *cReal, PRECISION *cImag, PRECISION *distance);
void fractalShow(int bufToggle);
void fractalCalculationEnd();
void calculationStopped();
bool isDiverged(int x, int y);
void setDiverged(int x, int y);
/* in Demo.cpp */
bool openDemo(char *path, char *file);
void startDemo();
/* in Color.cpp */
void setLPal(LogPalette &lPal, char *mapFile);
void saveLPal(LogPalette &lPal, char *mapFile);
void editColorFormInitialize();
void changeRGB();
void colorCycling(int cycle);
void changePalette();
void startCyclingAuto();
void cyclingInThread();
void cyclingPaint();
/* in Movie.cpp */
int  openMovieList();
int  getMovieHeader();
int  putMovieHeader();
int  getMovie1List(int n);
int  putMovie1List(int n);
int  getMovieList();
int  putMovieList();
void showListHeader();
void showListItem(int i);
void numberOfImagesToZoomInPower();
int  zoomInPowerToNumberOfImages(PRECISION zoomInPower);
void startMakeMovie();
void endMakeFrame();
void endMakeMovie();
void startMovie();
void movieInThread();
void moviePaint(int serialNo);
void endMovie();
/* in BitOperation.cpp */
void bitcpy(unsigned char *d, int dBit, unsigned char *s, int sBit, int n);
bool bitcmp(unsigned char *d, int dBit, unsigned char *s, int sBit, int n, int *sameN, int *revrN);
bool isbit(unsigned char *s, int sBit);
void bitset(unsigned char *s, int sBit, int offOn);
void ajustBitPos(unsigned char *s, int sBit, unsigned char **a, int *aBit);
div_t divide(int num, int denom);
bool hexaToBit(unsigned char *bitValue, char *hexa);
int  rightAlign(int bits);

#endif
