//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Common.h"
#include "Image.h"
#include "Control.h"
#include "FindingRoot.h"
#include "ReverseAnalysis.h"
#include "BodyAndHead.h"
#include "Spiral.h"
#include "ImageSize.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
bool openDemo(char *path, char *file)
{
    setDemoPath(path, file, EXT_DEMO);
    demo.fp = fopen(img.demoPath.path, "rb");
    if (demo.fp == NULL) {
        ShowMessage("Demo file not found");
        return false;
    }
    ReverseAnalysisForm->Hide();
    BodyAndHeadForm->Hide();
    SpiralForm->Hide();
    ControlForm->StatusClear();
    demo.onDemo       = 1;
    ImageForm->changeStaticMenu();
    demo.nonParFile   = 1;
    demo.infinite     = 0;
    demo.findingRoot  = 0;
    demo.showRoot     = 0;
    demo.caption[0]   = 0;
    demo.maxRetry     = -1;
    demo.imageSize[X] = -1;
    demo.imageSize[Y] = -1;
    demo.spiralTimes  = 500;
    setPath(img.demoPath.path, "", EXT_PAR, &demo.parPath);
    setPath(img.demoPath.path, "", EXT_SPR, &demo.sprPath);
    if (img.rootFindNonStop)  FindingRootForm->NonStopCheckBox->Checked = false;
    img.maxIterAuto = 0;
    ControlForm->ShowMaxIterAuto();
    return true;
}
//---------------------------------------------------------------------------
void startDemo()
{
    char *command[] = { "Infinite Demo",
                        "Show Root",
                        "Non Show Root",
                        "Form Caption = ",
                        "Immediate Form Caption = ",
                        "Max Retry = ",
                        "Image Size X = ",
                        "Image Size Y = ",
                        "Parameter File = ",
                        "Low Dimension",
                        "Color Cycling Start",
                        "Color Cycling Times = ",
                        "Color Cycling Reverse",
                        "Finding Root",
                        "Spiral Times = ",
                        "Spiral File = ",
                        0
                      };
    char buf[256], *p;
    int  i, len, dim;

    while (1) {
        memset(buf, 0, 256);
        p = fgets(buf, 255, demo.fp);
        buf[strlen(buf) - 1] = 0;
        buf[strlen(buf) - 1] = 0;
        if (p == NULL) {
            if ((demo.infinite == 0) || demo.nonParFile) {
                demo.onDemo = 0;
                ImageForm->changeStaticMenu();
                if (demo.findingRoot) {
                    FindingRootForm->NonStopCheckBox->Checked = false;
                    demo.findingRoot = 0;
                }
                fclose(demo.fp);
                return;
            }
            else  fseek(demo.fp, 0, SEEK_SET);
        }
        for (i = 0; command[i]; i++) {
            len = strlen(command[i]);
            if (strncmp(command[i], buf, len) == 0)  break;
        }
        switch (i) {
            case 0: // Infinite Demo
                demo.infinite = 1;
                break;
            case 1: // Show Root
                demo.showRoot = 1;
                break;
            case 2: // Non Show Root
                demo.showRoot = 0;
                break;
            case 3: // Form Caption =
                strcpy(demo.caption, "Mandelbrot Root Explorer ");
                strcat(demo.caption, buf + len);
                break;
            case 4: // Immediate Form Caption =
                strcpy(demo.caption, "Mandelbrot Root Explorer ");
                strcat(demo.caption, buf + len);
                ImageForm->Caption = demo.caption;
                break;
            case 5: // Max Retry =
                sscanf(buf + len, "%d", &demo.maxRetry);
                break;
            case 6: // Image Size X =
                sscanf(buf + len, "%d", &demo.imageSize[X]);
                break;
            case 7: // Image Size Y =
                sscanf(buf + len, "%d", &demo.imageSize[Y]);
                break;
            case 8: // Parameter File =
                setPath("", buf + len, EXT_PAR, &demo.parPath);
                if (openPar(demo.parPath.path, 0)) {
                    demo.nonParFile = 0;
                    par.imageSize[X] = init.sizeDefault[X];
                    par.imageSize[Y] = init.sizeDefault[Y];
                    if (demo.imageSize[X] > 0)  par.imageSize[X] = demo.imageSize[X];
                    if (demo.imageSize[Y] > 0)  par.imageSize[Y] = demo.imageSize[Y];
                    ImageSizeForm->ShowImageSizeXY();
                    if (demo.showRoot || par.nthRootAtRandom || img.rootFindNonStop) {
                        img.rootFindRetryN = 0;
                        RootChangedNonStop();
                        rootImageCalculationStart();
                    }
                    else {
                        ClearChanged();
                        fractalCalculationStart();
                    }
                    return;
                }
                break;
            case 9: // Low Dimension
                if (strlen(buf) == (unsigned int)len) {
                    low.fromDimension = 0;
                    low.toDimension = 3;
                }
                else {
                    sscanf(buf + len + 1, "%d", &dim);
                    if (dim < 0)   dim = 0;
                    if (dim > 16)  dim = 16;
                    low.fromDimension = dim;
                    low.toDimension = dim;
                }
                low.onDemo = 1;
                ImageForm->changeStaticMenu();
                low.dimension = low.fromDimension - 1;
                low.nthRoot = (1 << 20);
                startLow();
                return;
            case 10: // Color Cycling Start
                img.onZoom = 0;
                ImageForm->initDrawRect();
                startCyclingAuto();
                return;
            case 11: // Color Cycling Times =
                sscanf(buf + len, "%d", &img.cyclingTimes);
                break;
            case 12: // Color Cycling Reverse
                if (img.cyclingReverse == 0)  img.cyclingReverse = 1;
                else                          img.cyclingReverse = 0;
                break;
            case 13: // Finding Root
                if (img.rootFindNonStop == 0) {
                    FindingRootForm->NonStopCheckBox->Checked = true;
                    demo.findingRoot = 1;
                }
                break;
            case 14: // Spiral Times =
                sscanf(buf + len, "%d", &demo.spiralTimes);
                break;
            case 15: // Spiral File =
                setPath("", buf + len, EXT_SPR, &demo.sprPath);
                if (openSpiral(demo.sprPath.path)) {
                    ImageForm->SpiralClick(ImageForm);
                    if (img.dotLine != spr.dotLine) {
                        if (spr.dotLine == 0)  SpiralForm->DotRadioButtonClick(SpiralForm);
                        else                   SpiralForm->LineRadioButtonClick(SpiralForm);
                        ImageForm->BringToFront();
                    }
                    spiral(spr.topBoxCenter, spr.topBoxSpanX, spr.topBoxSpanY);
                    spiral(spr.nextBoxCenter, spr.nextBoxSpanX, spr.nextBoxSpanY);
                    spiralGo();
                    return;
                }
                break;
        }
    }
}
//---------------------------------------------------------------------------
