//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#define MAIN_FRACTAL_ROUTINE
#include "Common.h"
#include "Image.h"
#include "Control.h"
#include "FindingRoot.h"
#include "ReverseAnalysis.h"
#include "BodyAndHead.h"
#include "Spiral.h"
#include "ZoomInMovie.h"
#include "SelfSimilarityMovie.h"
#include "MovieList.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
void imageFormInitialize(int clear)
{
    ImageForm->initDrawRect();
    ImageForm->HorzScrollBar->Position = 0;
    ImageForm->VertScrollBar->Position = 0;
    if (img.formSizeAuto) {
        ImageForm->ClientWidth  = par.imageSize[X];  // need 2 times.
        ImageForm->ClientHeight = par.imageSize[Y];
        ImageForm->ClientWidth  = par.imageSize[X];
        ImageForm->ClientHeight = par.imageSize[Y];
        ImageForm->Image->Left = 0;
        ImageForm->Image->Top  = 0;
        ImageForm->Image->Width  = par.imageSize[X];
        ImageForm->Image->Height = par.imageSize[Y];
        ImageForm->AutoSize = true;
    }
    else {
        ImageForm->Image->Width  = par.imageSize[X];
        ImageForm->Image->Height = par.imageSize[Y];
        ImageForm->FormResize(ImageForm);
    }
    img.fractalBitmap->Width  = par.imageSize[X];
    img.fractalBitmap->Height = par.imageSize[Y];
    img.fractalBitmap->PixelFormat = pf8bit;
    img.fractalBitmap->Palette     = CreatePalette(&par.lPal.pal);
    if (((clear == 1) && (img.passMulti == 0)) || (clear == 2)) {
        img.fractalBitmap->Canvas->Brush->Color = clBlack;
        img.fractalBitmap->Canvas->FillRect(TRect(0, 0, par.imageSize[X], par.imageSize[Y]));
    }
    ImageForm->Image->Picture->Graphic = img.fractalBitmap;
    ImageForm->ShowAll();
}
//---------------------------------------------------------------------------
void fractalCalculationStart()
{
    PRECISION          spanX, spanY, sizeX, sizeY, shiftX, shiftY, sinA, cosA;
    complex<PRECISION> pCenter;
    PRECISION          pSpan[Y+1], defSpan[Y+1];

    pCenter  = par.pCenter;
    pSpan[X] = par.pSpan[X];
    pSpan[Y] = par.pSpan[Y];
    if ((img.reverseAnalysis == 0) || img.reverseAnalysisZoom || (img.onZoom == 0)) {
        if (demo.onDemo == 0) {
            ImageForm->Caption = "Mandelbrot Root Explorer (Calculating)";
            ImageForm->initDrawRect();
            if (img.passMulti == 0) {
                img.fractalBitmap->Canvas->Brush->Color = clBlack;
                img.fractalBitmap->Canvas->FillRect(TRect(0, 0, par.imageSize[X], par.imageSize[Y]));
                ImageForm->Image->Canvas->Draw(0, 0, img.fractalBitmap);
            }
            if (img.formSizeAuto) {
                ImageForm->ClientWidth  = par.imageSize[X];  // need 2 times
                ImageForm->ClientHeight = par.imageSize[Y];
                ImageForm->ClientWidth  = par.imageSize[X];
                ImageForm->ClientHeight = par.imageSize[Y];
            }
        }
        else {
            imageFormInitialize(1);
            ImageForm->Caption = demo.caption;
        }
        ControlForm->StartButton->Caption = "Stop";
    }
    if (par.formula == Mandelbrot) {
        par.c0Center  = complex<PRECISION>(0.0, 0.0);
        par.c0Span[X] = 0.0;
        par.c0Span[Y] = 0.0;
        if (img.onZoom == 0) {
            if (img.firstImage) {
                if (img.returnToMand == 0) {
                    defaultSpan(defSpan);
                    par.pSpan[X] = defSpan[X];
                    par.pSpan[Y] = defSpan[Y];
                    estimateMandelbrotSpan();
                    pSpan[X] = par.pSpan[X];
                    pSpan[Y] = par.pSpan[Y];
                }
                else {
                    pSpan[X] = img.returnToMandPSpan[X];
                    pSpan[Y] = img.returnToMandPSpan[Y];
                }
                img.firstImage = 0;
            }
        }
        else {
            if (img.moveCenter) {
                sizeX = (PRECISION)(par.imageSize[X] - 1);
                sizeY = (PRECISION)(par.imageSize[Y] - 1);
                shiftX = (2.0 * (PRECISION)img.zoomCenterX - sizeX) / (2.0 * sizeX) * pSpan[X];
                shiftY = -(2.0 * (PRECISION)img.zoomCenterY - sizeY) / (2.0 * sizeY) * pSpan[Y];
                sinA = sin(par.rotationAngle);
                cosA = cos(par.rotationAngle);
                pCenter += complex<PRECISION>(shiftX * cosA + shiftY * sinA, shiftY * cosA - shiftX * sinA);
                ChangeRootLabel(0);
                showCenterPosition(pCenter);
            }
            spanX = pSpan[X] * img.zoomMag;
            spanY = pSpan[Y] * img.zoomMag;
            if ((spanX > MANDELBROT_SPAN_LIMIT2) && (spanY > MANDELBROT_SPAN_LIMIT2)) {
                pSpan[X] = spanX;
                pSpan[Y] = spanY;
            }
            img.onZoom = 0;
            if (img.reverseAnalysis) {
                if (img.reverseAnalysisZoom == 0) {
                    ImageForm->initDrawRect();
                    if (img.reverseAnalysis == 1)       reverseAnalysis(pCenter, pSpan[X], pSpan[Y]);
                    else if (img.reverseAnalysis == 2)  bodyAndHead(pCenter, pSpan[X], pSpan[Y]);
                    else if (img.reverseAnalysis == 3)  spiral(pCenter, pSpan[X], pSpan[Y]);
                    return;
                }
            }
            if (img.zoomMag < 1.0)  ControlForm->StatusMessage("*** Zoom In ***");
            else                    ControlForm->StatusMessage("*** Zoom Out ***");
            img.maxIterAuto = 0;
            ControlForm->ShowMaxIterAuto();
        }
    }
    else {
        pSpan[X] = 0.0;
        pSpan[Y] = 0.0;
        if (img.onZoom == 0) {
            if (img.firstImage) {
                par.c0Center  = complex<PRECISION>(0.0, 0.0);
                defaultSpan(defSpan);
                par.c0Span[X] = defSpan[X];
                par.c0Span[Y] = defSpan[Y];
                img.firstImage = 0;
            }
        }
        else {
            if (img.moveCenter) {
                sizeX = (PRECISION)(par.imageSize[X] - 1);
                sizeY = (PRECISION)(par.imageSize[Y] - 1);
                shiftX = (2.0 * (PRECISION)img.zoomCenterX - sizeX) / (2.0 * sizeX) * par.c0Span[X];
                shiftY = -(2.0 * (PRECISION)img.zoomCenterY - sizeY) / (2.0 * sizeY) * par.c0Span[Y];
                sinA = sin(par.rotationAngle);
                cosA = cos(par.rotationAngle);
                par.c0Center += complex<PRECISION>(shiftX * cosA + shiftY * sinA, shiftY * cosA - shiftX * sinA);
            }
            spanX = par.c0Span[X] * img.zoomMag;
            spanY = par.c0Span[Y] * img.zoomMag;
            if ((spanX > MANDELBROT_SPAN_LIMIT2) && (spanY > MANDELBROT_SPAN_LIMIT2)) {
                par.c0Span[X] = spanX;
                par.c0Span[Y] = spanY;
            }
            img.onZoom = 0;
            if (img.zoomMag < 1.0)  ControlForm->StatusMessage("*** Zoom In ***");
            else                    ControlForm->StatusMessage("*** Zoom Out ***");
            img.maxIterAuto = 0;
            ControlForm->ShowMaxIterAuto();
        }
    }
    if (mov.onMakingMovie) {
        ControlForm->ShowMaxIter();
    }
    else if (img.rootFindNonStop && img.maxIterAuto) {
        par.maxIter = DEFAULT_MAXITER;
        if (img.rootFindRetryN > 4)  par.maxIter += (img.rootFindRetryN - 4) * MAXITER_PER_RETRY;
        ControlForm->ShowMaxIter();
    }
    else if (img.maxIterAuto) {
        par.maxIter = 512 + (par.dimension - 1) * MAXITER_PER_DIMENSION;
        ControlForm->ShowMaxIter();
    }
    par.pCenter  = pCenter;
    par.pSpan[X] = pSpan[X];
    par.pSpan[Y] = pSpan[Y];
    img.center = par.pCenter;
    ControlForm->ShowCenterX();
    ControlForm->ShowCenterY();
    if (par.formula == Mandelbrot) {
        img.span[X] = par.pSpan[X];
        img.span[Y] = par.pSpan[Y];
    }
    else {
        img.span[X] = par.c0Span[X];
        img.span[Y] = par.c0Span[Y];
    }
    ControlForm->ShowSpanX();
    ControlForm->ShowSpanY();
    img.passMultiNow = img.passMulti;
    img.pass = 0;
    if (img.passMultiNow) {
        img.passPixels = 1 << (POWER_FOR_PASS * (MALTI_PASS_NUMBER - 1));
        memset(img.diverged, 0, sizeof(img.diverged));
    }
    else {
        img.passPixels = 1;
    }
    img.fractalChanged = 0;
    if (mov.onMakingMovie == 0)  ImageForm->changeSaveMenu();
    img.onDrawing = Fractal;
    ImageForm->changeStaticMenu();
    img.threadTempStop  = 0;
    img.threadTerminate = 0;
    img.fractalCalculationCompleted = 0;
    img.threads = new TThreads(false);
}
//---------------------------------------------------------------------------
void defaultSpan(PRECISION *defSpan)
{
    if ((PRECISION)DEFAULT_IMAGE_SPAN_X / (PRECISION)DEFAULT_IMAGE_SPAN_Y >
         par.imageSize[X] / par.imageSize[Y]) {
         defSpan[X] = (PRECISION)DEFAULT_IMAGE_SPAN_X;
         defSpan[Y] = defSpan[X] * par.imageSize[Y] / par.imageSize[X];
    }
    else {
         defSpan[Y] = (PRECISION)DEFAULT_IMAGE_SPAN_Y;
         defSpan[X] = defSpan[Y] * par.imageSize[X] / par.imageSize[Y];
    }
}
//---------------------------------------------------------------------------
void estimateMandelbrotSpan()
{
    int                i, j, k, end;
    complex<PRECISION> deltaP, p;
    PRECISION          cReal, cImag, ratio, ratioMin, defSpan[Y+1];

    while (1) {
        ratioMin = 1.0;
        for (i = 0; i < 4; i++) {
            if (i < 2) {
                deltaP = complex<PRECISION>(par.pSpan[X] / par.imageSize[X], 0.0);
                if (i == 1)  deltaP = -deltaP;
                end = par.imageSize[X] / 2;
            }
            else {
                deltaP = complex<PRECISION>(0.0, par.pSpan[Y] / par.imageSize[Y]);
                if (i == 3)  deltaP = -deltaP;
                end = par.imageSize[Y] / 2;
            }
            for (j = 0; j < end; j++) {
                p = par.pCenter + (PRECISION)j * deltaP;
                cReal = 0.0;
                cImag = 0.0;
                k = fractalIteration(1024, p.real(), p.imag(), &cReal, &cImag);
                if (k != 1024)  break;
            }
            ratio = (PRECISION)j / (PRECISION)end;
            if (ratioMin > ratio)  ratioMin = ratio;
        }
        if ((ratioMin < 1.0) && (ratioMin > 0.05))  break;
        if (ratioMin >= 1.0) {
            par.pSpan[X] *= 9.0;
            par.pSpan[Y] *= 9.0;
        }
        else {
            par.pSpan[X] *= 0.05;
            par.pSpan[Y] *= 0.05;
            if ((par.pSpan[X] < MANDELBROT_SPAN_LIMIT) || (par.pSpan[Y] < MANDELBROT_SPAN_LIMIT)) {
                defaultSpan(defSpan);
                par.pSpan[X] = defSpan[X] * MANDELBROT_SPAN_LIMIT;
                par.pSpan[Y] = defSpan[Y] * MANDELBROT_SPAN_LIMIT;
                return;
            }
        }
    }
    par.pSpan[X] *= ratioMin / MANDELBROT_SPAN_RATIO;
    par.pSpan[Y] *= ratioMin / MANDELBROT_SPAN_RATIO;
}
//---------------------------------------------------------------------------
void fractalCalculationNextPass()
{
    img.onDrawing = Fractal;
    img.threads = new TThreads(false);
}
//---------------------------------------------------------------------------
static PRECISION log2, loglog2;
//---------------------------------------------------------------------------
void fractalCalculateInThread()
{
    PRECISION          cosA, sinA, c0SX, c0SY, pSX, pSY, cri, cReal, cImag, distance;
    complex<PRECISION> topC0, topP, deltaC0X, deltaC0Y, deltaPX, deltaPY, c0, p;
    complex<PRECISION> topXC0, topYC0, topXP, topYP;
    int                i, j, x, y, *calculatedN, paintCount, toggle, count;
    int                mask, maskp, xx, yy, deltaxx, deltayy, gotoFlag;
    BYTE               *ptr, color;
    complex<PRECISION> deltaC0, deltaP;
    BOOL               diverged = 0;

    img.fractalCalculatedX       = 0;
    img.fractalCalculatedY       = 0;
    img.fractalBufToggle         = 0;
    img.fractalBufCalculatedN[0] = 0;
    img.fractalBufCalculatedN[1] = 0;
    calculatedN = &img.fractalBufCalculatedN[img.fractalBufToggle];
    ptr = img.fractalBuf[img.fractalBufToggle];
    if (img.passMultiNow)  i = PAINT_COUNT / par.imageSize[X];
    else                   i = 1;
    paintCount = par.imageSize[X] * i;
    if (paintCount == 0)  paintCount = PAINT_COUNT;
    cosA = cos(par.rotationAngle);
    sinA = sin(par.rotationAngle);
    c0SX = par.c0Span[X];
    c0SY = par.c0Span[Y];
    pSX  = par.pSpan[X];
    pSY  = par.pSpan[Y];
    topC0  = complex<PRECISION>(-(c0SX * cosA - c0SY * sinA) / 2.0, (c0SX * sinA + c0SY * cosA) / 2.0);
    topC0 += par.c0Center;
    topP   = complex<PRECISION>(-(pSX * cosA - pSY * sinA) / 2.0, (pSX * sinA + pSY * cosA) / 2.0);
    topP  += par.pCenter;
    deltaC0X = complex<PRECISION>(c0SX * cosA, -c0SX * sinA) / (par.imageSize[X] - (PRECISION)1.0);
    deltaC0Y = complex<PRECISION>(-c0SY * sinA, -c0SY * cosA) / (par.imageSize[Y] - (PRECISION)1.0);
    deltaPX  = complex<PRECISION>(pSX * cosA, -pSX * sinA) / (par.imageSize[X] - (PRECISION)1.0);
    deltaPY  = complex<PRECISION>(-pSY * sinA, -pSY * cosA) / (par.imageSize[Y] - (PRECISION)1.0);
    mask = img.passPixels - 1;
    if (img.pass)  maskp = img.prevPassPixels - 1;
    topYC0 = topC0;
    topYP  = topP;
    if (par.outsideOption == 7) {
        log2 = logl((PRECISION)2.0);
        loglog2 = logl(log2);
    }
    for (y = 0; y < par.imageSize[Y]; y++, topYC0 += deltaC0Y, topYP += deltaPY) {
        if (y & mask)  continue;
        c0 = topYC0;
        p  = topYP;
        for (x = 0; x < par.imageSize[X]; x++, c0 += deltaC0X, p += deltaPX) {
            if (x & mask)  continue;
            if ((img.pass == 0) ||
                (((x & maskp) || (y & maskp)) && ((diverged = isDiverged(x, y)) == false))) {
                cReal = c0.real();
                cImag = c0.imag();
                if (par.outsideOption != 7) {
                    i = fractalIteration(par.maxIter, p.real(), p.imag(), &cReal, &cImag);
                }
                else {
                    fractalIterationWithDistance(par.maxIter, p.real(), p.imag(), &cReal, &cImag, &distance);
                    i = (int)(0.5 * distance);
                }
                if ((i == par.maxIter) && (par.outsideOption != 7)) {
                    i = 1;  // 0 is black
                    if (img.pass != MALTI_PASS_NUMBER - 1)  setDiverged(x, y);
                }
                else {
                    switch (par.outsideOption) {
                        case 1:
                            i += (int)cReal;
                            break;
                        case 2:
                            if (cReal >= 0)  i += (int)cReal;
                            else             i = 0;
                            break;
                        case 3:
                            i += (int)cImag;
                            break;
                        case 4:
                            if (cImag >= 0)  i += (int)cImag;
                            else             i = 0;
                            break;
                        case 5:
                            i += (int)(cReal * cImag);
                            break;
                        case 6:
                            cri = cReal * cImag;
                            if (cri >= 0)  i += (int)cri;
                            else           i = 0;
                            break;
                    }
                    i = i % (COLORS - 1);
                    if (i < 0)  i += (COLORS - 1);
                    if (i || (par.outsideOption%2) || (par.outsideOption == 0))  i++;
                }
                color = (BYTE)i;
            }
            else if (diverged) {
                color = 1;
                diverged = 0;
            }
            *(ptr++) = color;
            (*calculatedN)++;
            if (*calculatedN == paintCount)  {
                gotoFlag = 0;
                goto LabelMess;
LabelMessRet0:;
                ptr = img.fractalBuf[img.fractalBufToggle];
            }
            while (img.threadTempStop)  Sleep(50);
            if (img.threadTerminate) {
                calculationStopped();
                return;
            }
        }
    }
    if (*calculatedN) {
        gotoFlag = 1;
        goto LabelMess;
LabelMessRet1:;
    }
    if ((img.passMultiNow == 0) || (img.pass == MALTI_PASS_NUMBER - 1) || (par.outsideOption == 7)) {
        SendMessage(ImageForm->Handle, WM_FRACTAL_CALC_END, 0, 0);
        return;
    }
    // check inner part (stack algorithm ???)
    topYC0 = topC0;
    topYP  = topP;
    for (y = 0; y < par.imageSize[Y]; y++, topYC0 += deltaC0Y, topYP += deltaPY) {
        if (y & mask)  continue;
        topXC0 = topYC0;
        topXP  = topYP;
        for (x = 0; x < par.imageSize[X]; x++, topXC0 += deltaC0X, topXP += deltaPX) {
            if (x & mask)  continue;
            if ((x + 1 >= par.imageSize[X]) || (y + 1 >= par.imageSize[Y]))  continue;
            if (isDiverged(x + 1, y + 1))  continue;
            if (isDiverged(x, y) == false)  continue;
            if ((x + img.passPixels >= par.imageSize[X]) || (y + img.passPixels >= par.imageSize[Y]))  continue;
            if (isDiverged(x + img.passPixels, y) == false)  continue;
            if (isDiverged(x, y + img.passPixels) == false)  continue;
            if (isDiverged(x + img.passPixels, y + img.passPixels) == false)  continue;
            for (j = 0; j < 4; j++) { // box check
                switch (j) {
                case 0:
                case 2:
                    if (j == 0)  yy = y;
                    else         yy = y + img.passPixels;
                    xx = x + 1;
                    deltayy = 0;
                    deltaxx = 1;
                    deltaC0 = deltaC0X;
                    deltaP  = deltaPX;
                    break;
                case 1:
                case 3:
                    yy = y + 1;
                    if (j == 1)  xx = x;
                    else         xx = x + img.passPixels;
                    deltayy = 1;
                    deltaxx = 0;
                    deltaC0 = deltaC0Y;
                    deltaP  = deltaPY;
                    break;
                }
                c0 = topXC0;
                p  = topXP;
                for (i = 0; i < (yy - y); i++) {
                    c0 += deltaC0Y;
                    p  += deltaPY;
                }
                for (i = 0; i < (xx - x); i++) {
                    c0 += deltaC0X;
                    p  += deltaPX;
                }
                for (i = 1; i < img.passPixels; i++) {
                    if (isDiverged(xx, yy) == false) {
                        cReal = c0.real();
                        cImag = c0.imag();
                        if (fractalIteration(par.maxIter, p.real(), p.imag(), &cReal, &cImag) == par.maxIter)  setDiverged(xx, yy);
                        else                                                            break;
                    }
                    c0 += deltaC0;
                    p  += deltaP;
                    xx += deltaxx;
                    yy += deltayy;
                    while (img.threadTempStop)  Sleep(50);
                    if (img.threadTerminate) {
                        calculationStopped();
                        return;
                    }
                }
                if (i != img.passPixels)  break;
            }
            if (i == img.passPixels) { // paint in box
                for (i = 1; i < img.passPixels; i++) {
                    for (j = 1; j < img.passPixels; j++) {
                        setDiverged(x + j, y + i);
                    }
                }
            }
            while (img.threadTempStop)  Sleep(50);
            if (img.threadTerminate) {
                calculationStopped();
                return;
            }
        }
    }
    SendMessage(ImageForm->Handle, WM_FRACTAL_CALC_END, 0, 0);
    return;

LabelMess: // can't be subroutine. ???
    toggle = img.fractalBufToggle;
    if (img.fractalBufToggle)  img.fractalBufToggle = 0;
    else                       img.fractalBufToggle = 1;
    calculatedN = &img.fractalBufCalculatedN[img.fractalBufToggle];
    count = 0;
    while (*calculatedN) {
        Sleep(10);
        count++;
        if (count == 500)
            SendMessage(ImageForm->Handle, WM_FRACTAL_ON_PAINT, img.fractalBufToggle, 0);
        if (count == 1000) {
            ShowMessage("Hang up");
            calculationStopped();
            return;
        }
    }
    SendMessage(ImageForm->Handle, WM_FRACTAL_ON_PAINT, toggle, 0);
    if (gotoFlag == 0)  goto LabelMessRet0;
    else                goto LabelMessRet1;
}
//---------------------------------------------------------------------------
int fractalIteration(int maxIter, PRECISION pReal, PRECISION pImag,
        PRECISION *cReal, PRECISION *cImag)
{
    register int       iter;
    register PRECISION cR, cI, cReal2, cImag2;

    cR = *cReal;
    cI = *cImag;
    for (iter = 0; iter < maxIter; iter++) {
        cReal2 = cR * cR;
        cImag2 = cI * cI;
        cI = 2 * cR * cI + pImag;
        cR = cReal2 - cImag2 + pReal;
        if ((cReal2 + cImag2) > 4.0) break;
    }
    *cReal = cR;
    *cImag = cI;
    return iter;
}
//---------------------------------------------------------------------------
int  fractalIterationWithDistance(int maxIter, PRECISION pReal, PRECISION pImag,
         PRECISION *cReal, PRECISION *cImag, PRECISION *distance)
{
    register int       iter;
    register PRECISION cR, cI, cReal2, cImag2;
    PRECISION          pCR, pCI, dCR, dCI, dist;

    cR = *cReal;
    cI = *cImag;
    *distance = 0;
    for (iter = 0; iter < maxIter; iter++) {
        pCR = cR;
        pCI = cI;
        cReal2 = cR * cR;
        cImag2 = cI * cI;
        cI = 2 * cR * cI + pImag;
        cR = cReal2 - cImag2 + pReal;
        dCR = cR - pCR;
        dCI = cI - pCI;
        dist = sqrt(dCR * dCR + dCI * dCI);
        *distance += dist;
        if ((cReal2 + cImag2) > 4.0) {
            *distance -= dist * (logl(logl(cReal2 + cImag2)) - loglog2) / log2;
            break;
        }
    }
    if (iter == maxIter)  *distance *= 512.0 / iter;
    *cReal = cR;
    *cImag = cI;
    return iter;
}
//---------------------------------------------------------------------------
void fractalShow(int bufToggle)
{
    int   x, y, *calculatedN, i, j, xx, yy, passPixels2, iend, jend, flag, mask;
    BYTE  *ptr, *bptr;
    TRect rect;

    calculatedN = &img.fractalBufCalculatedN[bufToggle];
    if (*calculatedN == 0)  return;
    ptr = img.fractalBuf[bufToggle];
    x = img.fractalCalculatedX;
    passPixels2 = img.passPixels >> 1;
    if (img.pass)  mask = img.prevPassPixels - 1;
    if (img.passPixels == 1) {
        for (y = img.fractalCalculatedY; y < par.imageSize[Y]; ) {
            bptr = (BYTE *)img.fractalBitmap->ScanLine[y];
            for ( ; x < par.imageSize[X]; ) {
                if ((img.pass == 0) || ((x & mask) || (y & mask)))  bptr[x] = *ptr;
                ptr++;
                x++;
                (*calculatedN)--;
                if (*calculatedN == 0)  break;
            }
            if (x == par.imageSize[X]) {
                x = 0;
                y++;
            }
            if (*calculatedN == 0)  break;
        }
    }
    else {
        flag = 0;
        for (y = img.fractalCalculatedY; y < par.imageSize[Y]; ) {
            for ( ; x < par.imageSize[X]; ) {
                if ((img.pass == 0) || ((x & mask) || (y & mask))) {
                    iend = img.passPixels;
                    if (y + img.passPixels >= par.imageSize[Y]) {
                        iend += passPixels2;
                        flag = 1;
                    }
                    for (i = 0; i < iend; i++) {
                        yy = y + i - passPixels2;
                        if (yy < 0)  continue;
                        if (yy >= par.imageSize[Y])  break;
                        bptr = (BYTE *)img.fractalBitmap->ScanLine[yy];
                        jend = img.passPixels;
                        if (x + img.passPixels >= par.imageSize[X])  jend += passPixels2;
                        for (j = 0; j < jend; j++) {
                            xx = x + j - passPixels2;
                            if (xx < 0)  continue;
                            if (xx >= par.imageSize[X])  break;
                            bptr[xx] = *ptr;
                        }
                    }
                }
                ptr++;
                x += img.passPixels;
                (*calculatedN)--;
                if (*calculatedN == 0)  break;
            }
            if (x >= par.imageSize[X]) {
                x = 0;
                y += img.passPixels;
            }
            if (*calculatedN == 0)  break;
        }
    }
    rect.left   = 0;
    rect.right  = par.imageSize[X];
    rect.top    = img.fractalCalculatedY - passPixels2;
    if (rect.top < 0)  rect.top = 0;
    rect.bottom = y - passPixels2;
    if (x)  rect.bottom += img.passPixels;
    if (flag || (rect.bottom > par.imageSize[Y]))  rect.bottom = par.imageSize[Y];
    ImageForm->Image->Canvas->CopyRect(rect, img.fractalBitmap->Canvas, rect);
    img.fractalCalculatedX = x;
    img.fractalCalculatedY = y;
}
//---------------------------------------------------------------------------
void fractalCalculationEnd()
{
    int x, y, i, j;

    if (img.passMultiNow) {
        img.pass++;
        img.prevPassPixels = img.passPixels;
        img.passPixels >>= POWER_FOR_PASS;
        if (img.pass < MALTI_PASS_NUMBER) {
            fractalCalculationNextPass();
            return;
        }
    }
    if (img.saveImage) {
        savePar(0);
        saveImage(0);
    }
    if (img.rootFindNonStop) {
        rootImageCalculationStart();
        return;
    }
    if (!demo.onDemo)  ImageForm->Caption = "Mandelbrot Root Explorer (Classification and Auto View)";
    ControlForm->StartButton->Caption = "Start";
    img.onDrawing = Non;
    ImageForm->changeStaticMenu();
    if (low.onDemo) {
        y = par.imageSize[Y] / 2 - 1;
        for (i = 0; i < 3; i++, y++) {
            x = par.imageSize[X] / 2 - 1;
            for (j = 0; j < 3; j++, x++) {
                ImageForm->Image->Canvas->Pixels[x][y] = clRed;
            }
        }
        startLow();
        return;
    }
    if (demo.onDemo) {
        startDemo();
        return;
    }
    if (mov.onMakingMovie) {
        endMakeFrame();
        return;
    }
    if (par.nthRootAtRandom)  rootImageCalculationStart();
    else                      img.fractalCalculationCompleted = 1;
}
//---------------------------------------------------------------------------
void calculationStopped()
{
    if (img.onDrawing != Non) {
        if (img.onDrawing == SpiralShow)  spiralEndMessage();
        img.onDrawing = Non;
        if (low.onDemo) {
            low.onDemo = 0;
            ImageForm->changeStaticMenu();
        }
        if (demo.onDemo) {
            demo.onDemo = 0;
            ImageForm->changeStaticMenu();
            if (demo.findingRoot) {
                FindingRootForm->NonStopCheckBox->Checked = false;
                demo.findingRoot = 0;
            }
            fclose(demo.fp);
        }
        if (mov.onMakingMovie)  endMakeMovie();
        par.nthRootAtRandom = 0;
        ControlForm->ShowAtRandom();
        ImageForm->changeSaveMenu();
        img.cyclingAuto = 0;
        ImageForm->ShowCyclingAutoChecked();
        ImageForm->Caption = "Mandelbrot Root Explorer (Classification and Auto View)";
        ControlForm->StartButton->Caption = "Start";
        ImageForm->changeStaticMenu();
    }
}
//---------------------------------------------------------------------------
static int yPrev = -1;
static unsigned char *diverged;
//---------------------------------------------------------------------------
bool isDiverged(int x, int y)
{
    if (y != yPrev) {
        diverged = img.diverged[y];
        yPrev = y;
    }
    if (diverged[x >> 3] & (0x80 >> (x & 0x7)))  return true;
    return false;
}
//---------------------------------------------------------------------------
void setDiverged(int x, int y)
{
    if (y != yPrev) {
        diverged = img.diverged[y];
        yPrev = y;
    }
    diverged[x >> 3] |= (unsigned char)(0x80 >> (x & 0x7));
}
//---------------------------------------------------------------------------

