//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Common.h"
#include "Image.h"
#include "Control.h"
#include "ImageSize.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TImageSizeForm *ImageSizeForm;
//---------------------------------------------------------------------------
__fastcall TImageSizeForm::TImageSizeForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TImageSizeForm::FormShow(TObject *Sender)
{
    ShowAll();
}
//---------------------------------------------------------------------------
void __fastcall TImageSizeForm::S1RadioButtonClick(TObject *Sender)
{
    SetImageSize(0);
}
//---------------------------------------------------------------------------
void __fastcall TImageSizeForm::S2RadioButtonClick(TObject *Sender)
{
    SetImageSize(1);
}
//---------------------------------------------------------------------------
void __fastcall TImageSizeForm::S3RadioButtonClick(TObject *Sender)
{
    SetImageSize(2);
}
//---------------------------------------------------------------------------
void __fastcall TImageSizeForm::S4RadioButtonClick(TObject *Sender)
{
    SetImageSize(3);
}
//---------------------------------------------------------------------------
void __fastcall TImageSizeForm::S5RadioButtonClick(TObject *Sender)
{
    SetImageSize(4);
}
//---------------------------------------------------------------------------
void __fastcall TImageSizeForm::S6RadioButtonClick(TObject *Sender)
{
    SetImageSize(5);
}
//---------------------------------------------------------------------------
void __fastcall TImageSizeForm::XEditExit(TObject *Sender)
{
    SetImageSizeXY(X);
}
//---------------------------------------------------------------------------
void __fastcall TImageSizeForm::XEditKeyPress(TObject *Sender, char &Key)
{
    if (Key == '\r')  SetImageSizeXY(X);
}
//---------------------------------------------------------------------------
void __fastcall TImageSizeForm::YEditExit(TObject *Sender)
{
    SetImageSizeXY(Y);
}
//---------------------------------------------------------------------------
void __fastcall TImageSizeForm::YEditKeyPress(TObject *Sender, char &Key)
{
    if (Key == '\r')  SetImageSizeXY(Y);
}
//---------------------------------------------------------------------------
void __fastcall TImageSizeForm::SpanXButtonClick(TObject *Sender)
{
    img.span[X] = img.span[Y] * (PRECISION)par.imageSize[X] / (PRECISION)par.imageSize[Y];
    if (par.formula == Mandelbrot)  par.pSpan[X]  = img.span[X];
    else {
        par.c0Span[X] = img.span[X];
        if (img.returnToMand) {
            img.returnToMandPSpan[X] = img.returnToMandPSpan[Y] * (PRECISION)par.imageSize[X] / (PRECISION)par.imageSize[Y];
        }
    }
    ControlForm->ShowSpanX();
    FractalChanged();
}
//---------------------------------------------------------------------------
void __fastcall TImageSizeForm::SpanYButtonClick(TObject *Sender)
{
    img.span[Y] = img.span[X] * (PRECISION)par.imageSize[Y] / (PRECISION)par.imageSize[X];
    if (par.formula == Mandelbrot)  par.pSpan[Y]  = img.span[Y];
    else {
        par.c0Span[Y] = img.span[Y];
        if (img.returnToMand) {
            img.returnToMandPSpan[Y] = img.returnToMandPSpan[X] * (PRECISION)par.imageSize[Y] / (PRECISION)par.imageSize[X];
        }
    }
    ControlForm->ShowSpanY();
    FractalChanged();
}
//---------------------------------------------------------------------------
void __fastcall TImageSizeForm::HideButtonClick(TObject *Sender)
{
    Hide();
}
//---------------------------------------------------------------------------
void TImageSizeForm::SetImageSize(int button)
{
    int sizeXY[6][2] = { {200, 150}, {320, 200}, {640, 350},
                         {640, 480}, {800, 600}, {1024, 768} };

    if (ignore) {
        ignore = 0;
        return;
    }
    if (img.onDrawing != Non)  TerminateThread((void *)img.threads->Handle, 0);
    calculationStopped();
    if ((par.imageSize[X] * sizeXY[button][Y]) != (par.imageSize[Y] * sizeXY[button][X])) {
        SpanXButton->Enabled = true;
        SpanYButton->Enabled = true;
    }
    par.imageSize[X] = sizeXY[button][X];
    par.imageSize[Y] = sizeXY[button][Y];
    ShowImageSizeXY();
    FractalChanged();
    imageFormInitialize(2);
}
//---------------------------------------------------------------------------
void TImageSizeForm::SetImageSizeXY(int xy)
{
    int  imageSize, *s;

    try {
        if (xy == X)  imageSize = StrToInt(XEdit->Text);
        else          imageSize = StrToInt(YEdit->Text);
        if ((imageSize < IMAGE_SIZE_MIN) || (imageSize > IMAGE_SIZE_MAX)) {
            ShowMessage("Invalid number specified");
            if (imageSize < IMAGE_SIZE_MIN)  imageSize = IMAGE_SIZE_MIN;
            else                             imageSize = IMAGE_SIZE_MAX;
        }
        s = &par.imageSize[xy];
        if (*s != imageSize) {
            if (img.onDrawing != Non)  TerminateThread((void *)img.threads->Handle, 0);
            calculationStopped();
            SpanXButton->Enabled = true;
            SpanYButton->Enabled = true;
            *s = imageSize;
            FractalChanged();
            imageFormInitialize(2);
        }
    }
    catch(EConvertError *e) {
        ShowMessage("Invalid numeric specified");
    }
    ShowImageSizeXY();
}
//---------------------------------------------------------------------------
void TImageSizeForm::ShowImageSize()
{
    int sizeXY[6][2] = { {200, 150}, {320, 200}, {640, 350},
                         {640, 480}, {800, 600}, {1024, 768} };
    int i;
    TRadioButton *r[6] = { S1RadioButton,
                           S2RadioButton,
                           S3RadioButton,
                           S4RadioButton,
                           S5RadioButton,
                           S6RadioButton };

    for (i = 0; i < 6; i++) {
        if ((par.imageSize[X] == sizeXY[i][X]) && (par.imageSize[Y] == sizeXY[i][Y]))
            break;
    }
    if (i != 6) {
        if (r[i]->Checked == false)  ignore = 1;
        if (Visible)  r[i]->SetFocus();
        r[i]->Checked = true;
    }
}
//---------------------------------------------------------------------------
void TImageSizeForm::ShowImageSizeXY()
{
    char buf[32];

    sprintf(buf, "%d", par.imageSize[X]);
    XEdit->Text = buf;
    sprintf(buf, "%d", par.imageSize[Y]);
    YEdit->Text = buf;
}
//---------------------------------------------------------------------------
void TImageSizeForm::ShowAll()
{
    ShowImageSize();
    ShowImageSizeXY();
}
//---------------------------------------------------------------------------


