//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Common.h"
#include "Image.h"
#include "Control.h"
#include "FindingRoot.h"
#include "Option.h"
#include "ReverseAnalysis.h"
#include "BodyAndHead.h"
#include "Spiral.h"
#include "ImageSize.h"
#include "Zoom.h"
#include "MovieControl.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
void fractalInitialize()
{
    char path[MAXPATH];

    memset(&img, 0, sizeof(img));
    memset(&par, 0, sizeof(par));
    memset(&mov, 0, sizeof(mov));

    strcpy(img.startPath, ExtractFilePath(Application->ExeName).c_str());
    strcpy(path, img.startPath);
    strcat(path, DIR_DEMO);
    mkdir(path);
    setDemoPath(path, "", EXT_DEMO);
    strcpy(path, img.startPath);
    strcat(path, DIR_OPEN);
    mkdir(path);
    setOpenPath(path, "", EXT_PAR);
    strcpy(path, img.startPath);
    strcat(path, DIR_SAVE);
    mkdir(path);
    setSavePath(path, "", EXT_BMP);
    strcpy(path, img.startPath);
    strcat(path, DIR_SAVEAS);
    mkdir(path);
    setSaveAsPath(path, "", EXT_BMP);
    strcpy(path, img.startPath);
    strcat(path, DIR_MAP);
    mkdir(path);
    setMapPath(path, FILE_ROOT_MAP, EXT_MAP);
    setLPal(img.rootLPal, img.mapPath.path);
    setMapPath(path, FILE_FRACTAL_MAP, EXT_MAP);
    setLPal(par.lPal, img.mapPath.path);
    strcpy(path, img.startPath);
    strcat(path, DIR_MOVIE);
    mkdir(path);
    setMoviePath(path, "", "");

    img.fractalBitmap = new Graphics::TBitmap();
    img.rootBitmap    = new Graphics::TBitmap();
    img.colorBitmap   = new Graphics::TBitmap();
    img.rgbBitmap[0]  = new Graphics::TBitmap();
    img.rgbBitmap[1]  = new Graphics::TBitmap();
    img.rgbBitmap[2]  = new Graphics::TBitmap();
    if (OptionForm->GetInitial()) {
        if (init.fitToScreen) {
            OptionForm->GetFitToScreenSize(&init);
        }
    }
    else {
        OptionForm->SetDefaultOption();
    }
    resetParameters();
    imageFormInitialize(2);
    ImageForm->Visible = true;
    controlFormInitialize();
    ControlForm->Visible = true;
    ImageSizeForm->ShowAll();
    ZoomForm->ShowAll();
    MovieControlForm->ShowAll();

    if (init.demoPath[0]) {
        if (openDemo("", init.demoPath)) {
            startDemo();
            return;
        }
    }
    img.firstImage = 0;
    showCenterPosition(par.pCenter);
    fractalCalculationStart();
}
//---------------------------------------------------------------------------
void resetParameters()
{
    time_t t;

    RootChangedStop();
    img.rootChanged    = 0;
    img.zoomMode       = init.zoomModeDefault;
    img.passMulti      = init.passMultiDefault;
    img.onZoom         = 0;
    img.formSizeAuto   = 1;
    img.changeRGBIndex = 0;
    img.maxIterAuto    = 0;
    img.returnToMand   = 0;
    img.saveImage      = 0;
    img.saveImageExt   = Bmp;
    img.fractalCalculationCompleted = 0;
    img.rootFindNonStop      = 0;
    img.rootFindHiddenArea   = 1;
    img.rootFindWithFractal  = 0;
    img.rootFindZoomInPower  = ROOT_FIND_TRIAL_ZOOM_POWER;
    img.rootFindRetryMaxAuto = 1;
    img.rootFindRetryMax     = 5;
    img.rootFindRetryN       = 0;
    img.cyclingReverse = 0;
    img.cyclingAuto    = 0;
    img.cyclingTimes   = -1;
    img.zoomInPowerOnEdit = 0;
    img.maxRetryOnEdit = 0;
    img.movieReverse = 0;
    img.movieDelay   = 30;
    mov.onMakingMovie = 0;
    srand((unsigned) time(&t));
    strcpy(par.eyecat, EYECAT);
    par.parameterVersion  = PARAMETER_VERSION;
    par.parameterRevision = PARAMETER_REVISION;
    par.formula  = Mandelbrot;
    par.insideOption  = 0;
    par.outsideOption = 0;
    par.pCenter  = complex<PRECISION>(DEFAULT_IMAGE_CENTER_X, DEFAULT_IMAGE_CENTER_Y);
    par.pSpan[X] = DEFAULT_IMAGE_SPAN_X;
    par.pSpan[Y] = DEFAULT_IMAGE_SPAN_Y;
    par.c0Center  = complex<PRECISION>(0.0, 0.0);
    par.c0Span[X] = 0.0;
    par.c0Span[Y] = 0.0;
    par.maxIter  = DEFAULT_MAXITER;
    par.rotationAngle = 0.0;
    par.imageSize[X] = init.sizeDefault[X];
    par.imageSize[Y] = init.sizeDefault[Y];
    par.dimension = 0;
    par.periodicBitsNumber   = 1;
    par.periodicBitsValue[0] = 0x01;
    par.topBitsNumber        = 0;
    par.topBitsValue[0]      = 0x00;
    par.lastBitsNumber       = 0;
    par.lastBitsValue[0]     = 0x00;
    par.nthRootAtRandom      = 0;
    par.nthRootAtRandomKind  = 0;
    par.nthRootModifyBits    = 1;
    BitsToNthRoot();
    img.center  = par.pCenter;
    img.span[X] = par.pSpan[X];
    img.span[Y] = par.pSpan[Y];
    demo.onDemo = 0;
    low.onDemo  = 0;
    ControlForm->StatusClear();
    ImageForm->changeStaticMenu();
    ImageForm->changeSaveMenu();
}
//---------------------------------------------------------------------------
void resetAndShow()
{
    ReverseAnalysisForm->Hide();
    BodyAndHeadForm->Hide();
    SpiralForm->Hide();
    resetParameters();
    imageFormInitialize(2);
    ControlForm->ShowAll();
    FindingRootForm->ShowAll();
    ImageForm->ShowAll();
    ImageSizeForm->ShowAll();
    ZoomForm->ShowAll();
    MovieControlForm->ShowAll();
}
//---------------------------------------------------------------------------
void setDemoPath(char *path, char *file, char *ext)
{
    setPath(path, file, ext, &img.demoPath);
}
//---------------------------------------------------------------------------
void setOpenPath(char *path, char *file, char *ext)
{
    setPath(path, file, ext, &img.openPath);
}
//---------------------------------------------------------------------------
void setSavePath(char *path, char *file, char *ext)
{
    setPath(path, file, ext, &img.savePath);
}
//---------------------------------------------------------------------------
void setSaveAsPath(char *path, char *file, char *ext)
{
    setPath(path, file, ext, &img.saveAsPath);
}
//---------------------------------------------------------------------------
void setMapPath(char *path, char *file, char *ext)
{
    setPath(path, file, ext, &img.mapPath);
}
//---------------------------------------------------------------------------
void setMoviePath(char *path, char *file, char *ext)
{
    setPath(path, file, ext, &img.moviePath);
}
//---------------------------------------------------------------------------
void setPath(char *path, char *file, char *ext, FilePath *filePath)
{
    if (*path)  fnsplit(path, filePath->drive, filePath->dir, filePath->file, filePath->ext);
    if (*file)  strcpy(filePath->file, file);
    if (*ext)   strcpy(filePath->ext, ext);
    fnmerge(filePath->path, filePath->drive, filePath->dir, filePath->file, filePath->ext);
}
//---------------------------------------------------------------------------
bool openPar(char *path, int clearAtRandom)
{
    FILE *fp;

    fp = fopen(path, "rb");
    if (fp == NULL) {
        ShowMessage("Parameter file not found");
        return false;
    }
    if (fread(&par, sizeof(par), 1, fp) != 1) {
        fclose(fp);
        ShowMessage("Parameter file read error");
        return false;
    }
    fclose(fp);
    if (strcmp(par.eyecat, EYECAT)) {
        ShowMessage("Non parameter file");
        return false;
    }
    if (par.parameterVersion > PARAMETER_VERSION) {
        ShowMessage("Parameter version mismatch");
        return false;
    }
    if (clearAtRandom)  par.nthRootAtRandom = 0;
    if ((par.nthRootAtRandom == 0) && (img.rootFindNonStop == 0)) {
        img.center = par.pCenter;
        if (par.formula == Mandelbrot) {
            img.span[X] = par.pSpan[X];
            img.span[Y] = par.pSpan[Y];
        }
        else {
            img.span[X] = par.c0Span[X];
            img.span[Y] = par.c0Span[Y];
        }
        ChangeRootLabel(0);
        img.rootNumber = 1;
        img.rootP[0] = par.pCenter;
        showRootPosition(0);
    }
    ControlForm->ShowAll();
    FindingRootForm->ShowAll();
    ImageForm->ShowAll();
    ImageSizeForm->ShowAll();
    img.firstImage = 0;
    return true;
}
//---------------------------------------------------------------------------
bool loadImage()
{
    setOpenPath("", "", EXT_BMP);
    try {
        img.fractalBitmap->LoadFromFile(img.openPath.path);
        ImageForm->Image->Picture->Graphic = img.fractalBitmap;
        img.fractalCalculationCompleted = 1;
        return true;
    }
    catch (...) {
        return false;
    }
}
//---------------------------------------------------------------------------
void savePar(int saveMode)
{
    FILE *fp;
    char file[MAXFILE];

    if (saveMode == 0) {
        makeFileName(file);
        if (!mov.onMakingMovie) {
            setSavePath("", file, EXT_PAR);
        }
        else {
            setMoviePath("", file, EXT_PAR);
            return;
        }
        fp = fopen(img.savePath.path, "wb");
    }
    else {
        setSaveAsPath(ImageForm->SaveDialog->FileName.c_str(), "", EXT_PAR);
        fp = fopen(img.saveAsPath.path, "wb");
    }
    if (fp == NULL) {
        ShowMessage("Parameter file write error");
        return;
    }
    if (fwrite(&par, sizeof(par), 1, fp) != 1) {
        fclose(fp);
        ShowMessage("Parameter file write error");
        return;
    }
    fclose(fp);
}
//---------------------------------------------------------------------------
void saveImage(int saveMode)  // call savePar first
{
    char *path;
    char *ext[2] = { EXT_BMP, EXT_JPG };

    if (saveMode == 0) {
        if (!mov.onMakingMovie) {
            setSavePath("", "", ext[img.saveImageExt]);
            path = img.savePath.path;
        }
        else {
            setMoviePath("", "", ext[img.saveImageExt]);
            path = img.moviePath.path;
        }
    }
    else {
        setSaveAsPath("", "", ext[img.saveImageExt]);
        path = img.saveAsPath.path;
    }
    try {
        if (img.saveImageExt == Bmp)
            ImageForm->Image->Picture->SaveToFile(path);
        else {
            TJPEGImage *jpg = new TJPEGImage;
            jpg->Assign(img.fractalBitmap);
            jpg->SaveToFile(path);
            delete jpg;
        }
    }
    catch (...) {
        ShowMessage("Image file write error");
        return;
    }
}
//---------------------------------------------------------------------------
void makeFileName(char *file)
{
    time_t tp;
    char   *p, buf[32];
    int    i, n;

    if (!mov.onMakingMovie) {
        if (par.formula == Mandelbrot)  strcpy(file, "Mandelbrot ");
        else                            strcpy(file, "Julia ");
        time(&tp);
        strcat(file, ctime(&tp));
        n = strlen(file);
        p = file;
        for (i = 0; i < n; i++, p++) {
            if (*p == ':')   *p = '_';
            if (*p == '\n')  *p = 0;
        }
    }
    else {
        strcpy(file, img.movListFileName);
        sprintf(buf, "%.4d", mov.nowSerialNumber);
        strcat(file, buf);
    }
}
//---------------------------------------------------------------------------
bool openSpiral(char *path)
{
    FILE *fp;

    fp = fopen(path, "rb");
    if (fp == NULL) {
        ShowMessage("Spiral file read error");
        return false;
    }
    if (fread(&spr, sizeof(spr), 1, fp) != 1) {
        fclose(fp);
        ShowMessage("Spiral file read error");
        return false;
    }
    fclose(fp);
    return true;
}
//---------------------------------------------------------------------------
void saveSpiral()
{
    FILE *fp;

    setSaveAsPath(ImageForm->SaveDialog->FileName.c_str(), "", EXT_SPR);
    fp = fopen(img.saveAsPath.path, "wb");
    if (fp == NULL) {
        ShowMessage("Spiral file write error");
        return;
    }
    spr.dotLine = img.dotLine;
    if (fwrite(&spr, sizeof(spr), 1, fp) != 1) {
        fclose(fp);
        ShowMessage("Spiral file write error");
        return;
    }
    fclose(fp);
}
//---------------------------------------------------------------------------

