//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Common.h"
#include "Image.h"
#include "Control.h"
#include "ImageSize.h"
#include "ZoomInMovie.h"
#include "SelfSimilarityMovie.h"
#include "MovieList.h"
#include "MovieControl.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
int openMovieList()
{
    int  button;

    img.movListFp = fopen(img.moviePath.path, "rb");
    if (img.movListFp) {
        fclose(img.movListFp);
        button = Application->MessageBox("  List already exist. Overwrite ?", "", MB_YESNO);
        if (button != IDYES) return 1;
    }
    img.movListFp = fopen(img.moviePath.path, "w+b");
    if (img.movListFp == NULL)  return 1;
    return 0;
}
//---------------------------------------------------------------------------
int getMovieHeader()
{
    if (fseek(img.movListFp, 0, SEEK_SET))  return 1;
    if (fread(&mov, sizeof(mov), 1, img.movListFp) == 0)  return 1;
    return 0;
}
//---------------------------------------------------------------------------
int putMovieHeader()
{
    if (fseek(img.movListFp, 0, SEEK_SET))  return 1;
    if (fwrite(&mov, sizeof(mov), 1, img.movListFp) == 0)  return 1;
    return 0;
}
//---------------------------------------------------------------------------
int getMovie1List(int n)
{
    if (fseek(img.movListFp, sizeof(mov) + sizeof(list) * n, SEEK_SET))  return 1;
    if (fread(&list, sizeof(list), 1, img.movListFp) == 0)  return 1;
    return 0;
}
//---------------------------------------------------------------------------
int putMovie1List(int n)
{
    if (fseek(img.movListFp, sizeof(mov) + sizeof(list) * n, SEEK_SET))  return 1;
    if (fwrite(&list, sizeof(list), 1, img.movListFp) == 0)  return 1;
    return 0;
}
//---------------------------------------------------------------------------
int getMovieList()
{
    int i;

    if (getMovieHeader())  return 1;
    showListHeader();
    for (i = 0; i < mov.numberOfImages; i++) {
        if (getMovie1List(i))  return 1;
        showListItem(i);
    }
    return 0;
}
//---------------------------------------------------------------------------
int putMovieList()
{
    int i;
    PRECISION u, v, w, r, z, y, d;
    complex<PRECISION>  l, c;

    if (putMovieHeader())  return 1;
    showListHeader();
    if (mov.zoomOrSim == 0) {
        z = 1.0 / mov.zoomInPower;
        y = powl(z, (PRECISION)(mov.numberOfImages - 1.0));
        d = (z - 1.0) / (y - 1.0);
        l = 0;
        c = 0;
        r = 0;
        if (mov.par0.formula == Mandelbrot) {
            list.center   = mov.par0.pCenter;
            list.constant = 0;
        }
        else {
            list.center   = mov.par0.c0Center;
            list.constant = mov.par0.pCenter;
        }
        list.rotationAngle = mov.par0.rotationAngle;
        w = 1.0;
        for (i = 0; i < mov.numberOfImages; i++) {
            v = (PRECISION)i / (PRECISION)(mov.numberOfImages - 1);
            u = 1.0 - v;
            list.center   += l;
            if (mov.par0.formula == Mandelbrot) {
                list.span[X]   = mov.par0.pSpan[X] * w;
                list.span[Y]   = mov.par0.pSpan[Y] * w;
            }
            else {
                list.span[X]   = mov.par0.c0Span[X] * w;
                list.span[Y]   = mov.par0.c0Span[Y] * w;
            }
            list.constant += c;
            list.maxIter   = mov.par0.maxIter * u + mov.par1.maxIter * v;
            list.rotationAngle += r;
            w *= z;
            if (i == 0) {
                if (mov.par0.formula == Mandelbrot) {
                    l = (mov.par1.pCenter - mov.par0.pCenter) * d;
                    c = 0;
                }
                else {
                    l = (mov.par1.c0Center - mov.par0.c0Center) * d;
                    c = (mov.par1.pCenter - mov.par0.pCenter) * d;
                }
                r = (mov.par1.rotationAngle - mov.par0.rotationAngle) * d;
            }
            else {
                l *= z;
                c *= z;
                r *= z;
            }
            if (putMovie1List(i))  return 1;
            showListItem(i);
        }
    }
    else {
        for (i = 0; i < mov.numberOfImages; i++) {
            if (putMovie1List(i))  return 1;
            showListItem(i);
        }
    }
    return 0;
}
//---------------------------------------------------------------------------
void showListHeader()
{
    TStringGrid *s;

    s = MovieListForm->MovieListStringGrid;
    if (mov.par0.formula == Mandelbrot) {
        MovieListForm->Width = 660;
        s->Width = 637;
        s->ColCount = 7;
        s->RowCount = mov.numberOfImages + 1;
        s->ColWidths[0] = 50;
        s->ColWidths[1] = 112;
        s->ColWidths[2] = 112;
        s->ColWidths[3] = 112;
        s->ColWidths[4] = 112;
        s->ColWidths[5] = 50;
        s->ColWidths[6] = 60;
        s->Cells[0][0] = "Serial N";
        s->Cells[1][0] = "Center X";
        s->Cells[2][0] = "Center Y";
        s->Cells[3][0] = "Span X";
        s->Cells[4][0] = "Span Y";
        s->Cells[5][0] = "Max Iter";
        s->Cells[6][0] = "Rotation";
    }
    else {
        MovieListForm->Width = 874;
        s->Width = 862;
        s->ColCount = 9;
        s->RowCount = mov.numberOfImages + 1;
        s->ColWidths[0] = 50;
        s->ColWidths[1] = 112;
        s->ColWidths[2] = 112;
        s->ColWidths[3] = 112;
        s->ColWidths[4] = 112;
        s->ColWidths[5] = 112;
        s->ColWidths[6] = 112;
        s->ColWidths[7] = 50;
        s->ColWidths[8] = 60;
        s->Cells[0][0] = "Serial N";
        s->Cells[1][0] = "Center X";
        s->Cells[2][0] = "Center Y";
        s->Cells[3][0] = "Span X";
        s->Cells[4][0] = "Span Y";
        s->Cells[5][0] = "Constant X";
        s->Cells[6][0] = "Constant Y";
        s->Cells[7][0] = "Max Iter";
        s->Cells[8][0] = "Rotation";
    }
}
//---------------------------------------------------------------------------
void showListItem(int i)
{
    TStringGrid *s;
    char buf[64];

    s = MovieListForm->MovieListStringGrid;
    if (mov.nowSerialNumber > i)  strcpy(buf, "*");
    else  buf[0] = 0;
    s->Cells[0][i + 1] = buf + IntToStr(i);
    if (mov.par0.formula == Mandelbrot) {
        sprintf(buf, "%.15Lf", list.center.real());
        s->Cells[1][i + 1] = buf;
        sprintf(buf, "%.15Lf", list.center.imag());
        s->Cells[2][i + 1] = buf;
        sprintf(buf, "%.15Lf", list.span[X]);
        s->Cells[3][i + 1] = buf;
        sprintf(buf, "%.15Lf", list.span[Y]);
        s->Cells[4][i + 1] = buf;
        s->Cells[5][i + 1] = IntToStr(list.maxIter);
        sprintf(buf, "%.4Lf", list.rotationAngle * (PRECISION)180.0 / (PRECISION)M_PI);
        s->Cells[6][i + 1] = buf;
    }
    else {
        sprintf(buf, "%.15Lf", list.center.real());
        s->Cells[1][i + 1] = buf;
        sprintf(buf, "%.15Lf", list.center.imag());
        s->Cells[2][i + 1] = buf;
        sprintf(buf, "%.15Lf", list.span[X]);
        s->Cells[3][i + 1] = buf;
        sprintf(buf, "%.15Lf", list.span[Y]);
        s->Cells[4][i + 1] = buf;
        sprintf(buf, "%.15Lf", list.constant.real());
        s->Cells[5][i + 1] = buf;
        sprintf(buf, "%.15Lf", list.constant.imag());
        s->Cells[6][i + 1] = buf;
        s->Cells[7][i + 1] = IntToStr(list.maxIter);
        sprintf(buf, "%.4Lf", list.rotationAngle * (PRECISION)180.0 / (PRECISION)M_PI);
        s->Cells[8][i + 1] = buf;
    }
}
//---------------------------------------------------------------------------
void numberOfImagesToZoomInPower()
{
    if (mov.par0.formula == Mandelbrot)
        mov.zoomInPower = powl((mov.par0.pSpan[X] / mov.par1.pSpan[X]), (PRECISION)(1.0 / (mov.numberOfImages - 1.0)));
    else
        mov.zoomInPower = powl((mov.par0.c0Span[X] / mov.par1.c0Span[X]), (PRECISION)(1.0 / (mov.numberOfImages - 1.0)));
}
//---------------------------------------------------------------------------
int zoomInPowerToNumberOfImages(PRECISION zoomInPower)
{
    PRECISION n;
    int numberOfImages;

    if (mov.par0.formula == Mandelbrot)
        n = logl(mov.par0.pSpan[X] / mov.par1.pSpan[X]) / logl(zoomInPower) + 1.0;
    else
        n = logl(mov.par0.c0Span[X] / mov.par1.c0Span[X]) / logl(zoomInPower) + 1.0;
    numberOfImages = (int)n;
    if (n - numberOfImages > 0.5)  numberOfImages++;
    if (numberOfImages < 2)      return 1;
    if (numberOfImages > 10000)  return 2;
    mov.numberOfImages = numberOfImages;
    return 0;
}
//---------------------------------------------------------------------------
void startMakeMovie()
{
    char buf[64], m[64];

    mov.onMakingMovie = 1;
    getMovie1List(mov.nowSerialNumber);
    if (mov.par0.formula == Mandelbrot) {
        par.pCenter  = list.center;
        par.pSpan[X] = list.span[X];
        par.pSpan[Y] = list.span[Y];
    }
    else {
        par.pCenter   = list.constant;
        par.c0Center  = list.center;
        par.c0Span[X] = list.span[X];
        par.c0Span[Y] = list.span[Y];
    }
    par.maxIter = list.maxIter;
    par.rotationAngle = list.rotationAngle;
    img.reverseAnalysis = 0;
    img.onZoom = 0;
    img.maxIterAuto = 0;
    img.saveImage = 1;
    ControlForm->ShowAll();
    img.firstImage = 0;
    img.rootNumber = 1;
    img.rootP[0] = par.pCenter;
    showRootPosition(0);
    strcpy(m, "*** Movie(");
    sprintf(buf, "%d/", mov.nowSerialNumber + 1);
    strcat(m, buf);
    sprintf(buf, "%d) ***", mov.numberOfImages);
    strcat(m, buf);
    ControlForm->StatusMessage(m);
    fractalCalculationStart();
}
//---------------------------------------------------------------------------
void endMakeFrame()
{
    mov.nowSerialNumber++;
    showListItem(mov.nowSerialNumber - 1);
    mov.onMakingMovie = 0;
    putMovieHeader();
    if (mov.nowSerialNumber >= mov.numberOfImages)  endMakeMovie();
    else                                            startMakeMovie();
}
//---------------------------------------------------------------------------
void endMakeMovie()
{
    ZoomInMovieForm->Hide();
    SelfSimilarityMovieForm->Hide();
    MovieListForm->Hide();
    mov.onMakingMovie = 0;
    fclose(img.movListFp);
    img.movListFp = 0;
    img.saveImage = 0;
    ControlForm->ShowSaveImage();
    ImageForm->enableSave();
    ImageForm->enableOutside();
    ImageForm->enableColor();
    ImageForm->enableMakeZoomInMovie();
    ImageForm->enableMakeSelfSimilarityMovie();
    ZoomInMovieForm->StartButton->Caption = "Start";
    SelfSimilarityMovieForm->StartButton->Caption = "Start";
}
//---------------------------------------------------------------------------
void startMovie()
{
    par = mov.par0;
    imageFormInitialize(2);
    ControlForm->StartButton->Caption = "Stop";
    img.movieExt        = Bmp;
    img.onDrawing       = SlideShow;
    img.threadTempStop  = 0;
    img.threadTerminate = 0;
    img.threads         = new TThreads(false);
}
//---------------------------------------------------------------------------
void movieInThread()
{
    int i;

    if (img.movieReverse == 0)  i = 0;
    else                        i = mov.nowSerialNumber - 1;
    while (1) {
        img.moviePainted = 0;
        SendMessage(ImageForm->Handle, WM_MOVIE_PAINT, i, 0);
        Sleep(img.movieDelay);
        while (img.moviePainted == 0)  Sleep(5);
        while (img.threadTempStop) {
            img.onTempStop = 1;
            Sleep(50);
        }
        if (img.threadTerminate) {
            SendMessage(ImageForm->Handle, WM_MOVIE_PAINT, -1, 0);
            return;
        }
        if (img.movieReverse == 0) {
            i++;
            if (i == mov.nowSerialNumber)  i = 0;
        }
        else {
            i--;
            if (i < 0)  i = mov.nowSerialNumber - 1;
        }
    }
}
//---------------------------------------------------------------------------
void moviePaint(int serialNo)
{
    FilePath path;
    char file[MAXFILE], buf[32];
    int button;

    if (serialNo < 0) {
        endMovie();
        return;
    }
    path = img.moviePath;
    strcpy(file, img.movListFileName);
    sprintf(buf, "%.4d", serialNo);
    strcat(file, buf);
    try {
        if (img.movieExt == Bmp) {
            setPath("", file, EXT_BMP, &path);
            ImageForm->Image->Picture->LoadFromFile(path.path);
        }
        else {
            setPath("", file, EXT_JPG, &path);
            TJPEGImage *jpg = new TJPEGImage;
            jpg->LoadFromFile(path.path);
            img.fractalBitmap->Assign(jpg);
            ImageForm->Image->Picture->Graphic = img.fractalBitmap;
            delete jpg;
        }
    }
    catch (...) {
        try {
            if (img.movieExt == Bmp) {
                img.movieExt = Jpg;
                setPath("", file, EXT_JPG, &path);
                TJPEGImage *jpg = new TJPEGImage;
                jpg->LoadFromFile(path.path);
                img.fractalBitmap->Assign(jpg);
                ImageForm->Image->Picture->Graphic = img.fractalBitmap;
                delete jpg;
            }
            else {
                img.movieExt = Bmp;
                setPath("", file, EXT_BMP, &path);
                ImageForm->Image->Picture->LoadFromFile(path.path);
            }
        }
        catch (...) {
            if (img.movieSkipNoImage == 0) {
                button = Application->MessageBox("Image File not found! Skip?", "", MB_YESNO);
                if (button == IDYES)  img.movieSkipNoImage = 1;
                else {
                    endMovie();
                    return;
                }
            }
        }
    }
    img.moviePainted = 1;
}
//---------------------------------------------------------------------------
void endMovie()
{
    int movieReverse, movieDelay;

    movieReverse = img.movieReverse;
    movieDelay = img.movieDelay;
    resetAndShow();
    img.movieReverse = movieReverse;
    img.movieDelay = movieDelay;
    MovieControlForm->ShowMovieReverse();
    MovieControlForm->ShowMovieDelay();
    ImageForm->enableRoot();
    ImageForm->enableFractal();
    ImageForm->enableColor();
    MovieControlForm->StopButton->Enabled = false;
    MovieControlForm->Hide();
    img.firstImage = 0;
    showCenterPosition(par.pCenter);
}
//---------------------------------------------------------------------------
