//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Common.h"
#include "Image.h"
#include "Control.h"
#include "FindingRoot.h"
#include "ReverseAnalysis.h"
#include "BodyAndHead.h"
#include "Spiral.h"
#include "Zoom.h"
#include "WhichDialog.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
void controlFormInitialize()
{
    img.rootBitmap->Width  = ROOT_IMAGE_SIZE_X;
    img.rootBitmap->Height = ROOT_IMAGE_SIZE_Y;
    img.rootBitmap->PixelFormat = pf8bit;
    img.rootBitmap->Palette     = CreatePalette(&img.rootLPal.pal);
    img.rootBitmap->Canvas->Brush->Color = clBlack;
    img.rootBitmap->Canvas->FillRect(TRect(0, 0, ROOT_IMAGE_SIZE_X, ROOT_IMAGE_SIZE_Y));
    ControlForm->Image->Picture->Graphic = img.rootBitmap;
    ControlForm->ShowAll();
    FindingRootForm->ShowAll();
}
//---------------------------------------------------------------------------
void rootImageCalculationStart()
{
    float rnd;
    int   i;

    if ((!img.rootChanged && !par.nthRootAtRandom && !img.rootFindNonStop) || img.centerChanged) {
        if (img.fractalCalculationCompleted == 0) {
            if (img.centerChanged) {
                img.firstImage = 0;
                img.centerChanged = 0;
                img.spanChanged   = 0;
                par.pCenter = img.center;
                if (par.formula == Mandelbrot) {
                    par.pSpan[X] = img.span[X];
                    par.pSpan[Y] = img.span[Y];
                }
                else {
                    par.c0Span[X] = img.span[X];
                    par.c0Span[Y] = img.span[Y];
                }
            }
            ControlForm->StatusClear();
            fractalCalculationStart();
        }
        else {
            img.onZoom = 0;
            ImageForm->initDrawRect();
            img.cyclingAuto = 1;
            ImageForm->ShowCyclingAutoChecked();
            startCyclingAuto();
        }
        return;
    }
    ControlForm->StartButton->Caption = "Stop";
    if (img.rootFindRetryN == 0) {
        img.rootBitmap->Palette = CreatePalette(&img.rootLPal.pal);
        img.rootBitmap->Canvas->Brush->Color = clWhite;
        img.rootBitmap->Canvas->FillRect(TRect(0, 0, ROOT_IMAGE_SIZE_X, ROOT_IMAGE_SIZE_Y));
        ControlForm->Image->Picture->Graphic = img.rootBitmap;
        if (demo.onDemo && (demo.maxRetry >= 0))  img.rootFindRetryMax = demo.maxRetry;
        else if (img.rootFindRetryMaxAuto) {
            img.rootFindRetryMax = par.dimension / ROOT_FIND_TRIAL_DIM_PER_RETRY + 1;
            if (par.dimension > 20)  img.rootFindRetryMax++;
            if (par.dimension > 50)  img.rootFindRetryMax++;
        }
        FindingRootForm->ShowMaxRetry();
        img.rootPCenter = complex<PRECISION>(DEFAULT_ROOT_IMAGE_CENTER_X, DEFAULT_ROOT_IMAGE_CENTER_Y);
        if (par.nthRootAtRandom) {
            if (par.nthRootAtRandomKind == 0) {
                BitsToNthRoot();
                for (i = 0; i < par.nthRootModifyBits; i++) {
                    rnd = (float)rand() / (float)RAND_MAX;
                    ModifyOneBitOfNthRoot(rnd);
                }
            }
            else  setRandomNthRoot();
            ControlForm->ShowNthRoot();
        }
        img.rootNonStopFound   = 0;
        img.rootFindNowZoomInPower = 1.0;
    }
    else {
        img.rootPCenter = img.rootFindCenterP;
        if (img.rootPCenter.imag() < 0.0)  img.rootPCenter = conj(img.rootPCenter);
        img.rootFindNowZoomInPower *= img.rootFindZoomInPower;
    }
    if ((1.0 / img.rootFindNowZoomInPower) < MANDELBROT_SPAN_LIMIT2 * 10) {
        img.rootFindRetryN = img.rootFindRetryMax;
        rootImageCalculationEnd();
        return;
    }
    img.rootPSpan[X] = DEFAULT_ROOT_IMAGE_SPAN_X / img.rootFindNowZoomInPower;
    img.rootPSpan[Y] = DEFAULT_ROOT_IMAGE_SPAN_Y / img.rootFindNowZoomInPower;
    img.center = img.rootPCenter;
    ControlForm->ShowCenterX();
    ControlForm->ShowCenterY();
    img.span[X] = img.rootPSpan[X];
    img.span[Y] = img.rootPSpan[Y];
    ControlForm->ShowSpanX();
    ControlForm->ShowSpanY();
    if (!img.rootNonStopFound)  ChangeRootLabel(0);
    img.clickIgnore = 0;
    img.rootBufToggle   = 0;
    img.rootCalculatedY = 0;
    img.rootShowed[0]   = 1;
    img.rootShowed[1]   = 1;
    img.onDrawing       = Root;
    img.threadTempStop  = 0;
    img.threadTerminate = 0;
    img.fractalCalculationCompleted = 0;
    ImageForm->changeStaticMenu();
    img.threads         = new TThreads(false);
}
//---------------------------------------------------------------------------
void setRandomNthRoot()
{
    int i, imax;
    unsigned char mask[8] = { 0xff, 0x01, 0x03, 0x07, 0x0f, 0x1f, 0x3f, 0x7f };

    imax = par.dimension / 8;
    if (par.dimension % 8)  imax++;
    for(i = 0; i < imax; par.nthRoot[i++] = (unsigned char)rand());
    par.nthRoot[0] &= mask[par.dimension % 8];
}
//---------------------------------------------------------------------------
void rootImageCalculateInThread()
{
#define RED_MAX 2
    int                x, y, j, k, m, n, val, toggle, count;
    register int       i;
    int                maxHiddenDim, maxHiddenDimN, neighborMinHiddenDim, nextMaxHiddenDim;
    unsigned char      nthRoot, mk;
    complex<PRECISION> p, f, topP, deltaPX, deltaPY, rootP;
    PRECISION          norma, fReal, fImag, fReal2, fImag2, pReal, pImag, maxReal, neighborReal;
    BYTE               *ptr;
    NewtonStart        *newtonS;

    topP    = img.rootPCenter - complex<PRECISION>(img.rootPSpan[X], -img.rootPSpan[Y]) / (PRECISION)2.0;
    deltaPX = complex<PRECISION>(img.rootPSpan[X] / (ROOT_IMAGE_SIZE_X - 1), 0);
    deltaPY = -complex<PRECISION>(0, img.rootPSpan[Y] / (ROOT_IMAGE_SIZE_Y - 1));
    // find max hidden dimension
    maxHiddenDimN = 0;
    maxHiddenDim  = 0;
    for (y = 0; y < ROOT_IMAGE_SIZE_Y; y++) {
        p  = topP;
        p += (PRECISION)y * deltaPY;
        for (x = 0; x < ROOT_IMAGE_SIZE_X; x++, p += deltaPX) {
            if (par.dimension) {
                j       = (par.dimension - 1) / 8;
                nthRoot = par.nthRoot[j];
            }
            k  = 0;
            mk = 1;
            fReal = 0;
            fImag = 0;
            pReal = p.real();
            pImag = p.imag();
            for (i = 0; i < par.dimension; i++) {
                fReal2 = fReal * fReal;
                fImag2 = fImag * fImag;
                fImag = 2 * fReal * fImag + pImag;
                fReal = fReal2 - fImag2 + pReal;
                if ((fReal2 + fImag2) > 4.0)  break;
                if (nthRoot & mk) {
                    if (fReal > 0.0)  break;
                }
                else {
                    if (fReal < 0.0)  break;
                }
                k++;
                mk <<= 1;
                if (k > 7) {
                    k  = 0;
                    mk = 1;
                    j--;
                    nthRoot = par.nthRoot[j];
                }
            }
            img.hiddenDimension[y][x] = i;
            img.hiddenReal[y][x] = fabsl(fReal);
            if (i == maxHiddenDim)  maxHiddenDimN++;
            else if (i > maxHiddenDim) {
                maxHiddenDimN = 1;
                maxHiddenDim  = i;
            }
            while (img.threadTempStop)  Sleep(50);
            if (img.threadTerminate) {
                calculationStopped();
                return;
            }
        }
    }
    // rewind max hidden dimension to increase Newton start points
    if (maxHiddenDimN < 20) {
        while (1) {
            nextMaxHiddenDim = 0;
            for (y = 0; y < ROOT_IMAGE_SIZE_Y; y++) {
                for (x = 0; x < ROOT_IMAGE_SIZE_X; x++) {
                    if ((img.hiddenDimension[y][x] > nextMaxHiddenDim) && (img.hiddenDimension[y][x] < maxHiddenDim))
                        nextMaxHiddenDim = img.hiddenDimension[y][x];
                }
            }
            maxHiddenDim = nextMaxHiddenDim;
            for (y = 0; y < ROOT_IMAGE_SIZE_Y; y++) {
                for (x = 0; x < ROOT_IMAGE_SIZE_X; x++) {
                    if (img.hiddenDimension[y][x] == maxHiddenDim)
                        maxHiddenDimN++;
                }
            }
            if (maxHiddenDimN >= 20)  break;
        }
    }
    // find Newton start points
    maxReal = neighborReal = 0;
    for (x = 0; x < ROOT_IMAGE_SIZE_X; img.newtonStartX[x++].norm = 10000000.0);
    for (y = 0; y < ROOT_IMAGE_SIZE_Y; img.newtonStartY[y++].norm = 10000000.0);
    ptr = img.rootBuf[img.rootBufToggle];
    for (y = 0; y < ROOT_IMAGE_SIZE_Y; y++) {
        p  = topP;
        p += (PRECISION)y * deltaPY;
        for (x = 0; x < ROOT_IMAGE_SIZE_X; x++, p += deltaPX) {
            if (par.dimension && (img.hiddenDimension[y][x] >= maxHiddenDim)) {
                neighborMinHiddenDim = img.hiddenDimension[y][x];
                for (m = y - 1; m < y + 2; m++) {
                    if ((m < 0) || (m == ROOT_IMAGE_SIZE_Y))  continue;
                    for (n = x - 1; n < x + 2; n++) {
                        if ((n < 0) || (n == ROOT_IMAGE_SIZE_X))  continue;
                        if (neighborMinHiddenDim > img.hiddenDimension[m][n]) {
                            neighborMinHiddenDim = img.hiddenDimension[m][n];
                            neighborReal = img.hiddenReal[m][n];
                        }
                    }
                }
            }
            if (par.dimension && ((img.hiddenDimension[y][x] < maxHiddenDim) ||
                ((img.hiddenDimension[y][x] == maxHiddenDim) && (neighborMinHiddenDim >= maxHiddenDim)))) {
                  norma = 2;
            }
            else {  // only upper (max hidden dimension)
                f       = -p;
                j       = 0;
                nthRoot = par.nthRoot[j];
                k       = (par.dimension - 1) % 8;
                mk      = (unsigned char)(1 << k);
                for (i = 0; i < par.dimension; i++) {
                    f = sqrt(f);
                    if (nthRoot & mk)  f = -f;
                    f -= p;
                    k--;
                    mk >>= 1;
                    if (k < 0) {
                        k  = 7;
                        mk = (unsigned char)0x80;
                        j++;
                        nthRoot = par.nthRoot[j];
                    }
                }
                norma = sqrt(norm(f));
                if (norma < img.newtonStartX[x].norm) {
                    img.newtonStartX[x].norm = norma;
                    img.newtonStartX[x].p    = p;
                }
                if (norma < img.newtonStartY[y].norm) {
                    img.newtonStartY[y].norm = norma;
                    img.newtonStartY[y].p    = p;
                }
                if (neighborReal > maxReal) {  // for next retry (this is key step)
                    maxReal = neighborReal;
                    img.rootFindCenterP = p;
                }
            }
            if (img.rootFindHiddenArea) {
                if (img.hiddenDimension[y][x] != par.dimension)  val = (int)(img.hiddenReal[y][x] * 8.0) + RED_MAX;
                else                                             val = COLORS - 2 - RED_MAX;
            }
            else                                                 val = (int)(norma * 64.0);
            if (val <= RED_MAX)  val += 1; // RED_MAX : see rootDefault.map  1 : see setLPal()
            else {
                val %= COLORS - 1 - RED_MAX;
                val += 1 + RED_MAX;
            }
            while (img.threadTempStop)  Sleep(50);
            if (img.threadTerminate) {
                calculationStopped();
                return;
            }
            *(ptr++) = (BYTE)val;
        }
        toggle = img.rootBufToggle;
        img.rootShowed[toggle] = 0;
        if (img.rootBufToggle)  img.rootBufToggle = 0;
        else                    img.rootBufToggle = 1;
        ptr = img.rootBuf[img.rootBufToggle];
        count = 0;
        while (img.rootShowed[img.rootBufToggle] == 0) {
            Sleep(10);
            count++;
            if (count == 500)
                SendMessage(ControlForm->Handle, WM_ROOT_ON_PAINT, img.rootBufToggle, 0);
            if (count == 1000)
                ShowMessage("Hang up");
        }
        SendMessage(ControlForm->Handle, WM_ROOT_ON_PAINT, toggle, 0);
    }
    // find root by Newton method
    img.rootNumber = 0;
    for (i = 0; i < 2; i++) {
        if (i == 0) {
            j = ROOT_IMAGE_SIZE_Y - 1;
            m = -1;
            newtonS = img.newtonStartY;
        }
        else {
            j = 0;
            m = 1;
            newtonS = img.newtonStartX;
        }
        for ( ; ; j += m) {
            if (img.rootNumber >= ROOT_NUMBER_MAX)  break;
            if ((i == 0) && (j < 0))  break;
            if (i && (j >= ROOT_IMAGE_SIZE_X))  break;
            if ((newtonS + j)->norm > 1.8)  continue;
            if (getRootByNewtonMethod(par.dimension, par.nthRoot, &((newtonS + j)->p), &rootP)) {
                if (rootP.imag() < 0.0)  rootP = conj(rootP);
                for (k = 0; k < img.rootNumber; k++) {
                    if (sqrt(norm(rootP - img.rootP[k])) < 100.0 * NEWTON_EPSILON)  break;
                }
                if (k != img.rootNumber) continue;
                for (k = 0; k < img.rootNumber; k++) {
                    if (rootP.imag() > img.rootP[k].imag())  break;
                }
                memmove(&img.rootP[k + 1], &img.rootP[k], sizeof(complex<PRECISION>) * (img.rootNumber - k));
                img.rootP[k] = rootP;
                img.rootNumber++;
            }
            while (img.threadTempStop)  Sleep(50);
            if (img.threadTerminate) {
                calculationStopped();
                return;
            }
        }
    }
    while (img.threadTempStop)  Sleep(50);
    SendMessage(ControlForm->Handle, WM_ROOT_CALC_END, 0, 0);
}
//---------------------------------------------------------------------------
void rootImageShow(int bufToggle)
{
    int   x;
    BYTE  *ptr, *bptr;
    TRect rect;

    if (img.rootShowed[bufToggle] == 1)  return;
    ptr  = img.rootBuf[bufToggle];
    bptr = (BYTE *)img.rootBitmap->ScanLine[img.rootCalculatedY];
    for (x = 0; x < ROOT_IMAGE_SIZE_X; bptr[x++] = *(ptr++));
    rect.left   = 0;
    rect.top    = img.rootCalculatedY;
    rect.right  = ROOT_IMAGE_SIZE_X;
    rect.bottom = img.rootCalculatedY + 1;
    ControlForm->Image->Canvas->CopyRect(rect, img.rootBitmap->Canvas, rect);
    img.rootCalculatedY++;
    img.rootShowed[bufToggle] = 1;
}
//---------------------------------------------------------------------------
void rootImageCalculationEnd()
{
    int                i;
    char               xc[64], yc[64];
    char               *noRoot    = "*** Can't Find Root ***";
    char               *rootFound = "*** Root Found ***";
    char               *aster  = "*****************";
    TEdit *editX[8] = { WhichDlg->EditX0, WhichDlg->EditX1, WhichDlg->EditX2, WhichDlg->EditX3,
                        WhichDlg->EditX4, WhichDlg->EditX5, WhichDlg->EditX6, WhichDlg->EditX7 };
    TEdit *editY[8] = { WhichDlg->EditY0, WhichDlg->EditY1, WhichDlg->EditY2, WhichDlg->EditY3,
                        WhichDlg->EditY4, WhichDlg->EditY5, WhichDlg->EditY6, WhichDlg->EditY7 };

    if (img.rootNumber && (img.rootFindNonStop == 0)) {
        if (img.rootNumber > 1) {
            showRootPosition(-1);
            if (par.nthRootAtRandom || demo.onDemo)  i = rand() % img.rootNumber;
            else {
                WhichDlg->RadioButton0->Checked = true;
                img.rootWhich = 0;
                for (i = 0; i < 8; i++) {
                    if (i < img.rootNumber) {
                        sprintf(xc, "%.17Lf", img.rootP[i].real());
                        sprintf(yc, "%.17Lf", img.rootP[i].imag());
                        editX[i]->Text = xc;
                        editY[i]->Text = yc;
                    }
                    else {
                        editX[i]->Text = aster;
                        editY[i]->Text = aster;
                    }
                }
                WhichDlg->ShowModal();
                i = img.rootWhich;
            }
        }
        else  i = 0;
        showRootPosition(i);
        par.pCenter = img.rootP[i];
        img.center = par.pCenter;
        ControlForm->ShowCenterX();
        ControlForm->ShowCenterY();
        ChangeRootLabel(1);
        ControlForm->StatusMessage(rootFound);
    }
    else {
        if (img.rootNonStopFound && (img.rootNumber == 0)) {
            img.rootP[0] = img.rootNonStopCenterP;
            img.rootNumber = 1;
        }
        if (img.rootNumber) {
            showFindPosition();
            if (img.rootNonStopFound == 0) {
                img.rootNonStopFound = 1;
                img.rootNonStopCenterP = img.rootP[rand() % img.rootNumber];
            }
            img.rootFindCenterP = img.rootNonStopCenterP;
        }
        if (img.rootFindRetryN < img.rootFindRetryMax) {
            img.rootFindRetryN++;
            if (img.rootNumber) {
                ChangeRootLabel(1);
                strcpy(xc, "*** Found(");
                img.rootNumber = 0;
            }
            else  strcpy(xc, "*** Retry(");
            sprintf(yc, "%d/", img.rootFindRetryN);
            strcat(xc, yc);
            sprintf(yc, "%d) ***", img.rootFindRetryMax);
            strcat(xc, yc);
            ControlForm->StatusMessage(xc);
            if (img.rootFindWithFractal) {
                par.formula  = Mandelbrot;
                par.pCenter  = img.rootPCenter;
                par.pSpan[X] = img.rootPSpan[X];
                par.pSpan[Y] = img.rootPSpan[Y];
                if (par.pSpan[X] * (PRECISION)par.imageSize[Y] > par.pSpan[Y] * (PRECISION)par.imageSize[X])
                    par.pSpan[Y] = par.pSpan[X] * (PRECISION)par.imageSize[Y] / (PRECISION)par.imageSize[X];
                else
                    par.pSpan[X] = par.pSpan[Y] * (PRECISION)par.imageSize[X] / (PRECISION)par.imageSize[Y];
                img.rootChanged   = 0;
                img.centerChanged = 0;
                img.spanChanged   = 0;
                img.onZoom       = 0;
                img.firstImage   = 0;
                img.returnToMand = 0;
                fractalCalculationStart();
            }
            else  rootImageCalculationStart();
            return;
        }
        else {
            if (img.rootNumber) {
                ControlForm->StatusMessage(rootFound);
                img.rootNumber = 0;
            }
            else {
                ControlForm->StatusAlarm(noRoot);
            }
            img.rootFindRetryN = 0;
        }
    }
    if (img.rootNumber) {
        img.rootChanged   = 0;
        img.centerChanged = 0;
        img.spanChanged   = 0;
        img.rootFindRetryN = 0;
        img.onZoom       = 0;
        img.firstImage   = 1;
        img.returnToMand = 0;
        fractalCalculationStart();
        return;
    }
    if (low.onDemo) {
        startLow();
        return;
    }
    if (demo.onDemo && img.rootFindNonStop) {
        startDemo();
        return;
    }
    if (par.nthRootAtRandom) {
        rootImageCalculationStart();
        return;
    }
    ControlForm->StartButton->Caption = "Start";
    img.onDrawing = Non;
    ImageForm->changeStaticMenu();
}
//---------------------------------------------------------------------------
bool getRootByNewtonMethod(int dimension, unsigned char *nthRoot,
         complex<PRECISION> *newtonStartP, complex<PRECISION> *rootP)
{
    int                i, j, k, m, flag, count;
    unsigned char      n, mk;
    complex<PRECISION> p, f, fdash, s;
    PRECISION          val;

    p = *newtonStartP;
    flag = 0;
    count = 0;
    for (m = 0; m < NEWTON_REPEAT; m++) {
        f     = -p;
        fdash = -1;
        j     = 0;
        n     = nthRoot[j];
        k     = (dimension - 1) % 8;
        mk    = (unsigned char)(1 << k);
        for (i = 0; i < dimension; ) {
            s = sqrt(f);
            if (n & mk)  s = -s;
            f = s - p;
            if (norm(s) < 100.0 * NEWTON_EPSILON)  return false;
            fdash = fdash / ((PRECISION)2.0 * s) - (PRECISION)1.0;
            i++;
            k--;
            mk >>= 1;
            if (k < 0) {
                k  = 7;
                mk = (unsigned char)0x80;
                j++;
                n = nthRoot[j];
            }
        }
        val = sqrt(norm(f));
        if (val < NEWTON_EPSILON)  break;
        if (val < 100.0 * NEWTON_EPSILON)  count++;
        else                               count = 0;
        if (count > 2)  break;
        if (norm(fdash) < NEWTON_EPSILON) {
            flag = 1;
            break;
        }
        p -= f / fdash;
    }
    if ((m == NEWTON_REPEAT) || (flag))  return false;
    *rootP = p;
    return true;
}
//---------------------------------------------------------------------------
void showRootPosition(int colorRed)
{
    int i, x, y;
    char bmp[MAXPATH];
    TColor c;

    strcpy(bmp, img.startPath);
    strcat(bmp, "Mandelbrot.bmp");
    ControlForm->Image->Picture->LoadFromFile(bmp);
    for (i = 0; i < img.rootNumber; i++) {
        if (i == colorRed)  c = clRed;
        else                c = clGreen;
        x = ((img.rootP[i].real() - DEFAULT_ROOT_IMAGE_CENTER_X) / DEFAULT_ROOT_IMAGE_SPAN_X + 0.5) * ROOT_IMAGE_SIZE_X;
        y = (-(img.rootP[i].imag() - DEFAULT_ROOT_IMAGE_CENTER_Y) / DEFAULT_ROOT_IMAGE_SPAN_Y + 0.5) * ROOT_IMAGE_SIZE_Y;
        ControlForm->Image->Canvas->Pen->Color = c;
        ControlForm->Image->Canvas->Pen->Width = 3;
        ControlForm->Image->Canvas->Pen->Style = psSolid;
        ControlForm->Image->Canvas->MoveTo(x, y);
        ControlForm->Image->Canvas->LineTo(x, y);
    }
}
//---------------------------------------------------------------------------
void showCenterPosition(complex<PRECISION> center)
{
    int x, y;
    char bmp[MAXPATH];

    strcpy(bmp, img.startPath);
    strcat(bmp, "Mandelbrot.bmp");
    ControlForm->Image->Picture->LoadFromFile(bmp);
    x = ((center.real() - DEFAULT_ROOT_IMAGE_CENTER_X) / DEFAULT_ROOT_IMAGE_SPAN_X + 0.5) * ROOT_IMAGE_SIZE_X;
    y = (-(center.imag() - DEFAULT_ROOT_IMAGE_CENTER_Y) / DEFAULT_ROOT_IMAGE_SPAN_Y + 0.5) * ROOT_IMAGE_SIZE_Y;
    ControlForm->Image->Canvas->Pen->Color = clRed;
    ControlForm->Image->Canvas->Pen->Width = 3;
    ControlForm->Image->Canvas->Pen->Style = psSolid;
    ControlForm->Image->Canvas->MoveTo(x, y);
    ControlForm->Image->Canvas->LineTo(x, y);
}
//---------------------------------------------------------------------------
void showFindPosition()
{
    int i, x, y;
    TColor c = clRed;

    for (i = 0; i < img.rootNumber; i++) {
        x = ((img.rootP[i].real() - img.rootPCenter.real()) / img.rootPSpan[X] + 0.5) * ROOT_IMAGE_SIZE_X;
        y = (-(img.rootP[i].imag() - img.rootPCenter.imag()) / img.rootPSpan[Y] + 0.5) * ROOT_IMAGE_SIZE_Y;
        img.rootBitmap->Canvas->Pixels[x][y] = c;
        img.rootBitmap->Canvas->Pixels[x+1][y] = c;
        img.rootBitmap->Canvas->Pixels[x][y+1] = c;
        img.rootBitmap->Canvas->Pixels[x+1][y+1] = c;
        y = ((img.rootP[i].imag() + img.rootPCenter.imag()) / img.rootPSpan[Y] + 0.5) * ROOT_IMAGE_SIZE_Y;
        img.rootBitmap->Canvas->Pixels[x][y] = c;
        img.rootBitmap->Canvas->Pixels[x+1][y] = c;
        img.rootBitmap->Canvas->Pixels[x][y+1] = c;
        img.rootBitmap->Canvas->Pixels[x+1][y+1] = c;
    }
    ControlForm->Image->Picture->Graphic = img.rootBitmap;
}
//---------------------------------------------------------------------------
void reverseAnalysisFormInitialize()
{
    ReverseAnalysisForm->ZoomCheckBox->Checked = false;
    ReverseAnalysisForm->DimensionEdit->Text  = "";
    ReverseAnalysisForm->NthRootEdit->Text    = "";
    ReverseAnalysisForm->RootXEdit->Text      = "";
    ReverseAnalysisForm->RootYEdit->Text      = "";
    ReverseAnalysisForm->ResultMessage("");
}
//---------------------------------------------------------------------------
void reverseAnalysis(complex<PRECISION> boxCenter, PRECISION boxSpanX, PRECISION boxSpanY)
{
    int           revDimension, ret;
    unsigned char revNthRoot[REV_MAX_DIMENSION/8];
    char          buf[REV_MAX_DIMENSION/4+2];

    reverseAnalysisFormInitialize();
    ret = reverseCalculation(boxCenter, boxSpanX, boxSpanY, &revDimension, revNthRoot, &img.revRoot1);
    if ((ret == 1) || (ret == 2)) {
        if (ret == 1)  ReverseAnalysisForm->ResultAlarm("Cannot calculate dimension. (diverge z^2+p)");
        else           ReverseAnalysisForm->ResultAlarm("Cannot calculate dimension. (over 16384)");
        return;
    }
    sprintf(buf, "%d", revDimension);
    ReverseAnalysisForm->DimensionEdit->Text = buf;
    BitsToHexa(revDimension, revNthRoot, buf);
    ReverseAnalysisForm->NthRootEdit->Text = buf;
    if (ret == 4)  {
        ReverseAnalysisForm->RootXEdit->Text = "Root not found";
        ReverseAnalysisForm->RootYEdit->Text = "Root not found";
        ReverseAnalysisForm->ResultAlarm("Root not found. (Span too small ? Box too large ?)");
        return;
    }
    sprintf(buf, "%.17Lf", img.revRoot1.real());
    ReverseAnalysisForm->RootXEdit->Text = buf;
    sprintf(buf, "%.17Lf", img.revRoot1.imag());
    ReverseAnalysisForm->RootYEdit->Text = buf;
    if (ret == 0) {
        ReverseAnalysisForm->ResultMessage("Root found. Check OK.");
        img.reverseShowMode = 0;
    }
    else {
        ReverseAnalysisForm->ResultAlarm("Not in Box, may be ghost.");
        img.reverseShowMode = 1;
    }
    img.boxCenter = boxCenter;
    img.onDrawing = ReverseShow;
    img.threads   = new TThreads(false);
}
//---------------------------------------------------------------------------
int reverseCalculation(complex<PRECISION> boxCenter, PRECISION boxSpanX, PRECISION boxSpanY,
        int *revDimension, unsigned char *revNthRoot, complex<PRECISION> *root)
{
    int                 i, j, k;
    complex<PRECISION>  p[4], c[4], f, offset;
    PRECISION           real2, imag2, angle[4], diffa[4], totala;
    unsigned char       nthRoot, mk;

    memset(revNthRoot, 0, REV_MAX_DIMENSION/8);
    // If par.rotationAngle !=0, then p[i] is a little different from box.
    // But, there may be no problem.
    p[0] = boxCenter + complex<PRECISION>(- boxSpanX / 2.0, - boxSpanY / 2.0);
    p[1] = boxCenter + complex<PRECISION>(  boxSpanX / 2.0, - boxSpanY / 2.0);
    p[2] = boxCenter + complex<PRECISION>(  boxSpanX / 2.0,   boxSpanY / 2.0);
    p[3] = boxCenter + complex<PRECISION>(- boxSpanX / 2.0,   boxSpanY / 2.0);
    for (j = 0; j < 4; c[j++] = 0);
    for (i = 0; i < REV_MAX_DIMENSION; i++) {
        for (j = 0; j < 4; j++) {
            real2 = c[j].real() * c[j].real();
            imag2 = c[j].imag() * c[j].imag();
            c[j] = complex<PRECISION>(real2 - imag2, 2 * c[j].real() * c[j].imag()) + p[j];
            if ((real2 + imag2) > 1000.0)  return 1;
            angle[j] = arg(c[j]);
        }
        for (totala = j = 0; j < 4; j++) {
            k = j + 1;
            if (k == 4)  k = 0;
            diffa[j] = angle[k] - angle[j];
            if (fabsl(diffa[j]) > M_PI) {
                if (diffa[j] > 0)  diffa[j] = diffa[j] - 2 * M_PI;
                else               diffa[j] = diffa[j] + 2 * M_PI;
            }
            totala += diffa[j];
        }
        if (fabsl(totala) > M_PI / 10)  break;
    }
    if (i == REV_MAX_DIMENSION)  return 2;
    *revDimension = i;
    if (*revDimension) {
        j       = (*revDimension - 1) / 8;
        nthRoot = 0;
    }
    k  = 0;
    mk = 1;
    for (i = 0, f = 0; i < *revDimension; i++) {
        real2 = f.real() * f.real();
        imag2 = f.imag() * f.imag();
        f = complex<PRECISION>(real2 - imag2, 2 * f.real() * f.imag()) + boxCenter;
        if (f.real() < 0.0)  nthRoot |= mk;
        k++;
        mk <<= 1;
        if (k > 7) {
            revNthRoot[j] = nthRoot;
            k  = 0;
            mk = 1;
            j--;
            nthRoot = 0;
        }
    }
    if (k)  revNthRoot[0] = nthRoot;
    if (getRootByNewtonMethod(*revDimension, revNthRoot, &boxCenter, root)) {
        offset = *root - boxCenter;
        if ((fabsl(offset.real()) > boxSpanX / 2.0) || (fabsl(offset.imag()) > boxSpanY / 2.0))
              return 3;
        else  return 0;
    }
    else  return 4;
}
//---------------------------------------------------------------------------
void reverseShowInThread()
{
    complex<PRECISION> offset0, offset1, offset2, rotation;

    rotation = complex<PRECISION>(cos(par.rotationAngle), sin(par.rotationAngle));
    offset0  = img.revRoot1 - par.pCenter;
    offset1  = img.boxCenter - par.pCenter;
    offset0 *= rotation;
    offset1 *= rotation;
    img.reverseShowX0 = (offset0.real() / par.pSpan[X] + 0.5) * par.imageSize[X];
    img.reverseShowY0 = (- offset0.imag() / par.pSpan[Y] + 0.5) * par.imageSize[Y];
    if (img.reverseShowMode != 1) {
        img.reverseShowX1 = img.reverseShowX0;
        img.reverseShowY1 = img.reverseShowY0;
    }
    else {
        img.reverseShowX1 = (offset1.real() / par.pSpan[X] + 0.5) * par.imageSize[X];
        img.reverseShowY1 = (- offset1.imag() / par.pSpan[Y] + 0.5) * par.imageSize[Y];
    }
    if (img.reverseShowMode == 2) {
        offset2  = img.revRoot2 - par.pCenter;
        offset2 *= rotation;
        img.reverseShowX2 = (offset2.real() / par.pSpan[X] + 0.5) * par.imageSize[X];
        img.reverseShowY2 = (- offset2.imag() / par.pSpan[Y] + 0.5) * par.imageSize[Y];
    }
    SendMessage(ImageForm->Handle, WM_REVERSE_PAINT, 0, 0);
    Sleep(1500);
    SendMessage(ImageForm->Handle, WM_REVERSE_PAINT, 0, 0);
    img.onDrawing = Non;
}
//---------------------------------------------------------------------------
void reverseShow()
{
    ImageForm->Image->Canvas->Pen->Color = clWhite;
    if (img.reverseShowMode != 1)  ImageForm->Image->Canvas->Pen->Width = 8;
    else                           ImageForm->Image->Canvas->Pen->Width = 3;
    ImageForm->Image->Canvas->Pen->Style = psSolid;
    ImageForm->Image->Canvas->Pen->Mode  = pmXor;
    ImageForm->Image->Canvas->MoveTo(img.reverseShowX0, img.reverseShowY0);
    ImageForm->Image->Canvas->LineTo(img.reverseShowX1, img.reverseShowY1);
    if (img.reverseShowMode == 2) {
        ImageForm->Image->Canvas->MoveTo(img.reverseShowX2, img.reverseShowY2);
        ImageForm->Image->Canvas->LineTo(img.reverseShowX2, img.reverseShowY2);
    }
}
//---------------------------------------------------------------------------
void bodyAndHeadFormInitialize()
{
    BodyAndHeadForm->ZoomCheckBox->Checked = false;
    BodyAndHeadForm->BDimensionEdit->Text  = "";
    BodyAndHeadForm->BNthRootEdit->Text    = "";
    BodyAndHeadForm->BRootXEdit->Text      = "";
    BodyAndHeadForm->BRootYEdit->Text      = "";
    BodyAndHeadForm->HDimensionEdit->Text  = "";
    BodyAndHeadForm->HNthRootEdit->Text    = "";
    BodyAndHeadForm->HNthRoot1Edit->Text   = "";
    BodyAndHeadForm->HNthRoot2Edit->Text   = "";
    BodyAndHeadForm->HNthRoot3Edit->Text   = "";
    BodyAndHeadForm->HRootXEdit->Text      = "";
    BodyAndHeadForm->HRootYEdit->Text      = "";
    BodyAndHeadForm->ResultMessage("");
}
//---------------------------------------------------------------------------
void bodyAndHead(complex<PRECISION> boxCenter, PRECISION boxSpanX, PRECISION boxSpanY)
{
    int           bodyDimension, headDimension, ret, midBit, button, nthN;
    int           leftReversed, rightReversed;
    unsigned char bodyNthRoot[REV_MAX_DIMENSION/8], headNthRoot[REV_MAX_DIMENSION/8];
    unsigned char bodyNthRootLeft[REV_MAX_DIMENSION/8], bodyNthRootRight[REV_MAX_DIMENSION/8];
    char          buf[REV_MAX_DIMENSION/4+2], buf1[32];
    complex<PRECISION> root;

    if (img.headNotFound == 0) {
        bodyAndHeadFormInitialize();
        ret = reverseCalculation(boxCenter, boxSpanX, boxSpanY, &bodyDimension, bodyNthRoot, &root);
        if ((ret == 1) || (ret == 2)) {
            if (ret == 1)  BodyAndHeadForm->ResultAlarm("Cannot calculate dimension. (diverge z^2+p)");
            else           BodyAndHeadForm->ResultAlarm("Cannot calculate dimension. (over 16384)");
            return;
        }
        sprintf(buf, "%d", bodyDimension);
        BodyAndHeadForm->BDimensionEdit->Text = buf;
        BitsToHexa(bodyDimension, bodyNthRoot, buf);
        BodyAndHeadForm->BNthRootEdit->Text = buf;
        if (ret == 4)  {
            BodyAndHeadForm->BRootXEdit->Text = "Root not found";
            BodyAndHeadForm->BRootYEdit->Text = "Root not found";
            BodyAndHeadForm->ResultAlarm("Body : Root not found. (Span too small ? Box too large ?)");
            return;
        }
        sprintf(buf, "%.17Lf", root.real());
        BodyAndHeadForm->BRootXEdit->Text = buf;
        sprintf(buf, "%.17Lf", root.imag());
        BodyAndHeadForm->BRootYEdit->Text = buf;
        img.revRoot1 = root;
        if (ret) {
            BodyAndHeadForm->ResultAlarm("Body : Not in Box, may be ghost.");
            img.reverseShowMode = 1;
            img.boxCenter = boxCenter;
            img.onDrawing = ReverseShow;
            img.threads   = new TThreads(false);
            return;
        }
        ret = getHeadRoot(boxCenter, boxSpanX, boxSpanY, bodyDimension, bodyNthRoot,
                          &midBit, headNthRoot, &root);
        if (ret) {
            if (ret == 1)  BodyAndHeadForm->ResultAlarm("Head : Dimension over. (over 16384)");
            else {
                button = Application->MessageBox("  Head not found. Check head ?\n  If Yes, then please enclose head with box and click inside.", "", MB_YESNO);
                if (button == IDNO)  return;
                img.save1Dimension = bodyDimension;
                nthN = bodyDimension / 8;
                if (bodyDimension % 8)  nthN++;
                memcpy(img.save1NthRoot, bodyNthRoot, nthN);
                BodyAndHeadForm->ResultAlarm("Please enclose head with box and click inside.");
                img.headNotFound = 1;
            }
            return;
        }
        headDimension = 2 * bodyDimension + 1;
        sprintf(buf, "%d", headDimension);
        BodyAndHeadForm->HDimensionEdit->Text = buf;
        BitsToHexa(headDimension, headNthRoot, buf);
        BodyAndHeadForm->HNthRootEdit->Text = buf;
        if (midBit == 0)  BodyAndHeadForm->HNthRoot2Edit->Text = "0";
        else              BodyAndHeadForm->HNthRoot2Edit->Text = "1";
        BodyAndHeadForm->HNthRoot1Edit->Font->Color = clWindowText;
        BodyAndHeadForm->HNthRoot3Edit->Font->Color = clWindowText;
        BodyAndHeadForm->HNthRoot1Edit->Text = BodyAndHeadForm->BNthRootEdit->Text;
        BodyAndHeadForm->HNthRoot3Edit->Text = BodyAndHeadForm->BNthRootEdit->Text;
        sprintf(buf, "%.17Lf", root.real());
        BodyAndHeadForm->HRootXEdit->Text = buf;
        sprintf(buf, "%.17Lf", root.imag());
        BodyAndHeadForm->HRootYEdit->Text = buf;
        BodyAndHeadForm->ResultMessage("Head's root found. Check OK.");
    }
    else {
        img.headNotFound = 0;
        ret = reverseCalculation(boxCenter, boxSpanX, boxSpanY, &headDimension, headNthRoot, &root);
        if ((ret == 1) || (ret == 2)) {
            if (ret == 1)  BodyAndHeadForm->ResultAlarm("Cannot calculate dimension. (diverge z^2+p)");
            else           BodyAndHeadForm->ResultAlarm("Cannot calculate dimension. (over 16384)");
            return;
        }
        bodyDimension = img.save1Dimension;
        nthN = bodyDimension / 8;
        if (bodyDimension % 8)  nthN++;
        memcpy(bodyNthRoot, img.save1NthRoot, nthN);
        sprintf(buf, "%d", headDimension);
        BodyAndHeadForm->HDimensionEdit->Text = buf;
        BitsToHexa(headDimension, headNthRoot, buf);
        BodyAndHeadForm->HNthRootEdit->Text = buf;
        if (headDimension != (2 * bodyDimension + 1)) {
            BodyAndHeadForm->ResultAlarm("Head : Really ? Span too small ?");
            return;
        }
        if (ret == 3) {
            BodyAndHeadForm->ResultAlarm("Head : Not in Box, may be ghost.");
            return;
        }
        splitHeadNthRoot(bodyDimension, bodyNthRoot, headNthRoot,
                         &leftReversed, &rightReversed, &midBit, bodyNthRootLeft, bodyNthRootRight);
        if (midBit == 0)  BodyAndHeadForm->HNthRoot2Edit->Text = "0";
        else              BodyAndHeadForm->HNthRoot2Edit->Text = "1";
        if (leftReversed == 0)   BodyAndHeadForm->HNthRoot1Edit->Font->Color = clWindowText;
        else                     BodyAndHeadForm->HNthRoot1Edit->Font->Color = clRed;
        if (rightReversed == 0)  BodyAndHeadForm->HNthRoot3Edit->Font->Color = clWindowText;
        else                     BodyAndHeadForm->HNthRoot3Edit->Font->Color = clRed;
        BitsToHexa(bodyDimension, bodyNthRootLeft, buf);
        BodyAndHeadForm->HNthRoot1Edit->Text = buf;
        BitsToHexa(bodyDimension, bodyNthRootRight, buf);
        BodyAndHeadForm->HNthRoot3Edit->Text = buf;
        if (ret == 4)  {
            BodyAndHeadForm->HRootXEdit->Text = "Root not found";
            BodyAndHeadForm->HRootYEdit->Text = "Root not found";
            BodyAndHeadForm->ResultAlarm("Head : Root not found. (Span too small ? Box too large ?)");
            return;
        }
        sprintf(buf, "%.17Lf", root.real());
        BodyAndHeadForm->HRootXEdit->Text = buf;
        sprintf(buf, "%.17Lf", root.imag());
        BodyAndHeadForm->HRootYEdit->Text = buf;
        if ((leftReversed == 0) && (rightReversed == 0)) {
            BodyAndHeadForm->ResultMessage("Head's root found. Check OK.");
        }
        else {
            strcpy(buf, "Head found. ");
            if (leftReversed) {
                strcat(buf, "Left ");
                sprintf(buf1, "%d", leftReversed);
                strcat(buf, buf1);
                strcat(buf, " Bit Reversed. ");
            }
            if (rightReversed) {
                strcat(buf, "Right ");
                sprintf(buf1, "%d", rightReversed);
                strcat(buf, buf1);
                strcat(buf, " Bit Reversed.");
            }
            BodyAndHeadForm->ResultAlarm(buf);
        }
    }
    img.reverseShowMode = 2;
    img.revRoot2  = root;
    img.boxCenter = boxCenter;
    img.onDrawing = ReverseShow;
    img.threads   = new TThreads(false);
    return;
}
//---------------------------------------------------------------------------
int getHeadRoot(complex<PRECISION> boxCenter, PRECISION boxSpanX, PRECISION boxSpanY,
        int bodyDimension, unsigned char *bodyNthRoot,
        int *midBit, unsigned char *headNthRoot, complex<PRECISION> *headRoot)
{
    int headDimension, i, bBit, hBit, mBit;

    headDimension = 2 * bodyDimension + 1;
    if (headDimension > REV_MAX_DIMENSION)  return 1;
    headNthRoot[0] = 0;
    bBit = rightAlign(bodyDimension);
    hBit = rightAlign(headDimension);
    bitcpy(headNthRoot, hBit, bodyNthRoot, bBit, -bodyDimension);
    mBit = hBit - bodyDimension;
    hBit = mBit - 1;
    bitcpy(headNthRoot, hBit, bodyNthRoot, bBit, -bodyDimension);
    for (i = 0; i < 2; i++) {
        bitset(headNthRoot, mBit, i);
        if (getRootInBox(boxCenter, boxSpanX, boxSpanY, 4, 1.0, headDimension, headNthRoot, headRoot))  break;
    }
    if (i == 2)  return 2;
    *midBit = i;
    return 0;
}
//---------------------------------------------------------------------------
void splitHeadNthRoot(int bodyDimension, unsigned char *bodyNthRoot, unsigned char *headNthRoot,
         int *leftReversed, int *rightReversed,
         int *midBit, unsigned char *bodyNthRootLeft, unsigned char *bodyNthRootRight)
{
    int headDimension, bBit, hBit, mBit, sameN;

    headDimension = 2 * bodyDimension + 1;
    bodyNthRootRight[0] = 0;
    bodyNthRootLeft[0]  = 0;
    bBit = rightAlign(bodyDimension);
    hBit = rightAlign(headDimension);
    bitcpy(bodyNthRootRight, bBit, headNthRoot, hBit, -bodyDimension);
    mBit = hBit - bodyDimension;
    hBit = mBit - 1;
    bitcpy(bodyNthRootLeft, bBit, headNthRoot, hBit, -bodyDimension);
    if (isbit(headNthRoot, hBit) == true)  *midBit = 1;
    else                                   *midBit = 0;
    bitcmp(bodyNthRoot, bBit, bodyNthRootRight, bBit, -bodyDimension, &sameN, rightReversed);
    bitcmp(bodyNthRoot, bBit, bodyNthRootLeft, bBit, -bodyDimension, &sameN, leftReversed);
}
//---------------------------------------------------------------------------
bool getRootInBox(complex<PRECISION> boxCenter, PRECISION boxSpanX, PRECISION boxSpanY,
         int mesh, PRECISION allowance, int dimension, unsigned char *nthRoot, complex<PRECISION> *root)
{
    int                i, j;
    complex<PRECISION> newtonStartP, top, deltX, deltY, offset;

    if (mesh == 1) {
        top   = boxCenter;
    }
    else {
        top   = boxCenter - complex<PRECISION>(boxSpanX, boxSpanY) / (PRECISION)2.0;
        deltX = complex<PRECISION>(boxSpanX / (mesh - 1), 0.0);
        deltY = complex<PRECISION>(0.0, boxSpanY / (mesh - 1));
    }
    for (i = 0; i < mesh; i++) {
        newtonStartP = top + (PRECISION)i * deltY;
        for (j = 0; j < mesh; j++, newtonStartP += deltX) {
            if (getRootByNewtonMethod(dimension, nthRoot, &newtonStartP, root)) {
                offset = *root - boxCenter;
                if ((fabsl(offset.real()) > boxSpanX / 2.0 * allowance) ||
                    (fabsl(offset.imag()) > boxSpanY / 2.0 * allowance))
                    continue;
                return true;
            }
        }
    }
    return false;
}
//---------------------------------------------------------------------------
void spiralFormInitialize()
{
    SpiralForm->SaveButton->Enabled   = false;
    SpiralForm->GoButton->Caption     = "Go";
    SpiralForm->GoButton->Enabled     = false;
    SpiralForm->ZoomCheckBox->Checked = false;
    SpiralForm->TDimensionEdit->Text  = "";
    SpiralForm->TNthRootEdit->Text    = "";
    SpiralForm->TNthRoot1Edit->Text   = "";
    SpiralForm->TNthRoot3Edit->Text   = "";
    SpiralForm->TRootXEdit->Text      = "";
    SpiralForm->TRootYEdit->Text      = "";
    SpiralForm->NDimensionEdit->Text  = "";
    SpiralForm->NNthRootEdit->Text    = "";
    SpiralForm->NNthRoot1Edit->Text   = "";
    SpiralForm->NNthRoot2Edit->Text   = "";
    SpiralForm->NNthRoot3Edit->Text   = "";
    SpiralForm->NRootXEdit->Text      = "";
    SpiralForm->NRootYEdit->Text      = "";
    SpiralForm->GDimensionEdit->Text  = "";
    SpiralForm->GNthRootEdit->Text    = "";
    SpiralForm->GInsertBitsEdit->Text = "";
    SpiralForm->GInsertEdit->Text     = "";
    SpiralForm->GTimesEdit->Text      = "";
    SpiralForm->GRootXEdit->Text      = "";
    SpiralForm->GRootYEdit->Text      = "";
    SpiralForm->ResultMessage("");
}
//---------------------------------------------------------------------------
void spiral(complex<PRECISION> boxCenter, PRECISION boxSpanX, PRECISION boxSpanY)
{
    int           topDimension, nextDimension, ret, nthN, *d;
    unsigned char topNthRoot[REV_MAX_DIMENSION/8], nextNthRoot[REV_MAX_DIMENSION/8], *n;
    char          buf[REV_MAX_DIMENSION/4+2];
    complex<PRECISION> root;
    TEdit         *ed, *en, *ex, *ey;

    topDimension  = img.save1Dimension;
    nextDimension = img.save2Dimension;
    if (topDimension > 0) {
        nthN = topDimension / 8;
        if (topDimension % 8)  nthN++;
        memcpy(topNthRoot, img.save1NthRoot, nthN);
    }
    if (nextDimension > 0) {
        nthN = nextDimension / 8;
        if (nextDimension % 8)  nthN++;
        memcpy(nextNthRoot, img.save2NthRoot, nthN);
    }
    if (img.topNext == 0) {
        spr.topBoxCenter = boxCenter;
        spr.topBoxSpanX  = boxSpanX;
        spr.topBoxSpanY  = boxSpanY;
        d  = &topDimension;
        n  = topNthRoot;
        ed = SpiralForm->TDimensionEdit;
        en = SpiralForm->TNthRootEdit;
        ex = SpiralForm->TRootXEdit;
        ey = SpiralForm->TRootYEdit;
        SpiralForm->TDimensionEdit->Text = "";
        SpiralForm->TNthRootEdit->Text   = "";
        SpiralForm->TNthRoot1Edit->Text  = "";
        SpiralForm->TNthRoot3Edit->Text  = "";
        SpiralForm->TRootXEdit->Text     = "";
        SpiralForm->TRootYEdit->Text     = "";
    }
    else {
        spr.nextBoxCenter = boxCenter;
        spr.nextBoxSpanX  = boxSpanX;
        spr.nextBoxSpanY  = boxSpanY;
        d  = &nextDimension;
        n  = nextNthRoot;
        ed = SpiralForm->NDimensionEdit;
        en = SpiralForm->NNthRootEdit;
        ex = SpiralForm->NRootXEdit;
        ey = SpiralForm->NRootYEdit;
        SpiralForm->NDimensionEdit->Text = "";
        SpiralForm->NNthRootEdit->Text   = "";
        SpiralForm->NNthRoot1Edit->Text  = "";
        SpiralForm->NNthRoot2Edit->Text  = "";
        SpiralForm->NNthRoot3Edit->Text  = "";
        SpiralForm->NRootXEdit->Text     = "";
        SpiralForm->NRootYEdit->Text     = "";
    }
    SpiralForm->GDimensionEdit->Text  = "";
    SpiralForm->GNthRootEdit->Text    = "";
    SpiralForm->GInsertBitsEdit->Text = "";
    SpiralForm->GInsertEdit->Text     = "";
    SpiralForm->GTimesEdit->Text      = "";
    SpiralForm->GRootXEdit->Text      = "";
    SpiralForm->GRootYEdit->Text      = "";
    SpiralForm->ResultMessage("");
    ret = reverseCalculation(boxCenter, boxSpanX, boxSpanY, d, n, &root);
    if ((ret == 1) || (ret == 2)) {
        if (img.topNext == 0)  strcpy(buf, "Top : ");
        else                   strcpy(buf, "Next : ");
        if (ret == 1)  strcat(buf, "Cannot calculate dimension. (diverge z^2+p)");
        else           strcat(buf, "Cannot calculate dimension. (over 16384)");
        SpiralForm->ResultAlarm(buf);
        return;
    }
    sprintf(buf, "%d", *d);
    ed->Text = buf;
    BitsToHexa(*d, n, buf);
    en->Text = buf;
    if (ret == 4)  {
        ex->Text = "Root not found";
        ey->Text = "Root not found";
        if (img.topNext == 0)  strcpy(buf, "Top : ");
        else                   strcpy(buf, "Next : ");
        strcat(buf, "Root not found. (Span too small ? Box too large ?)");
        SpiralForm->ResultAlarm(buf);
        return;
    }
    sprintf(buf, "%.17Lf", root.real());
    ex->Text = buf;
    sprintf(buf, "%.17Lf", root.imag());
    ey->Text = buf;
    img.revRoot1 = root;
    if (img.topNext == 0)  img.topRoot  = root;
    else                   img.nextRoot = root;
    if (ret) {
        if (img.topNext == 0)  strcpy(buf, "Top : ");
        else                   strcpy(buf, "Next : ");
        strcat(buf, "Not in Box, may be ghost.");
        SpiralForm->ResultAlarm(buf);
        img.reverseShowMode = 1;
        img.boxCenter = boxCenter;
        img.onDrawing = ReverseShow;
        img.threads   = new TThreads(false);
        return;
    }
    nthN = *d / 8;
    if (*d % 8)  nthN++;
    if (img.topNext == 0) {
        img.save1Dimension = *d;
        memcpy(img.save1NthRoot, n, nthN);
    }
    else {
        img.save2Dimension = *d;
        memcpy(img.save2NthRoot, n, nthN);
    }
    if (img.topNext == 0)  SpiralForm->NextRadioButtonClick(SpiralForm);
    else                   SpiralForm->TopRadioButtonClick(SpiralForm);
    if ((topDimension >= 0) && (nextDimension >= 0)) {
        ret = splitSpiralNthRoot(topDimension, topNthRoot, nextDimension, nextNthRoot,
                                 &img.leftBit, &img.insBit, &img.rightBit, &img.insCount,
                                 img.leftNthRoot, img.insNthRoot, img.rightNthRoot);
        if (ret) {
            SpiralForm->GoButton->Enabled = false;
            SpiralForm->ResultAlarm("Spiral ?");
        }
        else {
            BitsToHexa(img.leftBit, img.leftNthRoot, buf);
            SpiralForm->TNthRoot1Edit->Text = buf;
            SpiralForm->NNthRoot1Edit->Text = buf;
            sprintf(buf, "%d", img.insBit);
            SpiralForm->GInsertBitsEdit->Text = buf;
            BitsToHexa(img.insBit, img.insNthRoot, buf);
            SpiralForm->NNthRoot2Edit->Text = buf;
            SpiralForm->GInsertEdit->Text   = buf;
            BitsToHexa(img.rightBit, img.rightNthRoot, buf);
            SpiralForm->TNthRoot3Edit->Text = buf;
            SpiralForm->NNthRoot3Edit->Text = buf;
            SpiralForm->SaveButton->Enabled = true;
            SpiralForm->GoButton->Enabled   = true;
            if (demo.onDemo == 0)  SpiralForm->ResultAlarm("Please push Go Button.");
        }
    }
    if (demo.onDemo) {
        spiralDrawRect(boxCenter, boxSpanX, boxSpanY);
    }
    else {
        img.reverseShowMode = 0;
        img.onDrawing = ReverseShow;
        img.threads   = new TThreads(false);
    }
}
//---------------------------------------------------------------------------
int splitSpiralNthRoot(int topDimension, unsigned char *topNthRoot,
        int nextDimension, unsigned char *nextNthRoot,
        int *leftBit, int *insBit, int *rightBit, int *insCount,
        unsigned char *leftNthRoot, unsigned char *insNthRoot, unsigned char *rightNthRoot)
{
    int diffDimension, tBit, nBit, lBit, iBit, rBit, dum0, dum1, i;

    diffDimension = nextDimension - topDimension;
    if (diffDimension <= 0)  return 1;
    rightNthRoot[0] = 0;
    insNthRoot[0]   = 0;
    leftNthRoot[0]  = 0;
    tBit = rightAlign(topDimension);
    nBit = rightAlign(nextDimension);
    bitcmp(nextNthRoot, nBit, topNthRoot, tBit, -topDimension, rightBit, &dum0);
    iBit = rightAlign(diffDimension);
    bitcpy(insNthRoot, iBit, nextNthRoot, nBit - *rightBit, -diffDimension);
    *insBit = diffDimension;
    for (i = 0; i < *rightBit; i += diffDimension) {
        if (bitcmp(insNthRoot, iBit, topNthRoot, tBit - *rightBit + i + diffDimension,
                   -diffDimension, &dum0, &dum1) == false)  break;
    }
    *insCount  = i / diffDimension;
    *rightBit -= i;
    rBit = rightAlign(*rightBit);
    bitcpy(rightNthRoot, rBit, topNthRoot, tBit, -*rightBit);
    tBit -= *rightBit;
    nBit -= *rightBit;
    nBit -= *insBit;
    if (bitcmp(nextNthRoot, nBit, topNthRoot, tBit, -(topDimension - *rightBit), leftBit, &dum0) == false)  return 1;
    lBit = rightAlign(*leftBit);
    bitcpy(leftNthRoot, lBit, topNthRoot, tBit, -*leftBit);
    return 0;
}
//---------------------------------------------------------------------------
void spiralDrawRect(complex<PRECISION> boxCenter, PRECISION boxSpanX, PRECISION boxSpanY)
{
    TPoint point[5];
    complex<PRECISION> offset, rotation;
    int    centerX, centerY, widthX, widthY;

    rotation = complex<PRECISION>(cos(par.rotationAngle), sin(par.rotationAngle));
    offset  = boxCenter - par.pCenter;
    offset *= rotation;
    centerX = (offset.real() / par.pSpan[X] + 0.5) * par.imageSize[X];
    centerY = (- offset.imag() / par.pSpan[Y] + 0.5) * par.imageSize[Y];
    widthX  = boxSpanX / par.pSpan[X] * par.imageSize[X] / 2;
    widthY  = boxSpanY / par.pSpan[Y] * par.imageSize[Y] / 2;
    point[0] = TPoint(centerX - widthX, centerY + widthY);
    point[1] = TPoint(centerX - widthX, centerY - widthY);
    point[2] = TPoint(centerX + widthX, centerY - widthY);
    point[3] = TPoint(centerX + widthX, centerY + widthY);
    point[4] = TPoint(centerX - widthX, centerY + widthY);
    ImageForm->Image->Canvas->Pen->Color = clWhite;
    ImageForm->Image->Canvas->Pen->Width = 1;
    ImageForm->Image->Canvas->Pen->Style = psSolid;
    ImageForm->Image->Canvas->Pen->Mode  = pmCopy;
    ImageForm->Image->Canvas->Polyline(point, 5);
}
//---------------------------------------------------------------------------
void spiralGo()
{
    if (img.onDrawing) {
        img.threadTerminate = 1;
        if (img.onDrawing != SpiralShow)  SpiralForm->ResultAlarm("Thread stopped.");
    }
    else {
        ControlForm->StartButton->Caption = "Stop";
        ImageForm->disableCyclingAuto();
        SpiralForm->ResultMessage("");
        SpiralForm->GoButton->Caption = "Stop";
        img.threadTempStop  = 0;
        img.threadTerminate = 0;
        img.onDrawing = SpiralShow;
        img.threads   = new TThreads(false);
    }
}
//---------------------------------------------------------------------------
void spiralShowInThread()
{
// This routine is coded from few samples.
#define MESH_MAX    7
#define ALLOWANCE   2.5
#define SPAN_COEF   2.0
#define INI_D_RATIO 0.2
#define DIST_COEF   4
#define MAX_REV_INSED 10
    int                ret, sleepT, dum;
    int                mesh, revBit, revEnd, i, k;
    int                prevLeft, prevInsed, prevIns, prevRight;
    int                sBit, offOn, dotLine, insBit, insedBit, maxRevInsed;
    int                rBit, nBit, topBit, *prevBit, *nRev;
    complex<PRECISION> offset, rotation, newtonStart, prevRoot, distance;
    PRECISION          span, sDistance, dDistance, curDistRatio, prevDistRatio;
    unsigned char      leftNthRoot[REV_MAX_DIMENSION/8];
    unsigned char      insedNthRoot[REV_MAX_DIMENSION/8];
    unsigned char      insNthRoot[REV_MAX_DIMENSION/8];
    unsigned char      rightNthRoot[REV_MAX_DIMENSION/8];
    unsigned char      work[REV_MAX_DIMENSION/8];
    unsigned char      *pNth, *qNth;

    memcpy(leftNthRoot, img.leftNthRoot, img.leftBit / 8 + ((img.leftBit % 8) ? 1 : 0));
    memcpy(insNthRoot, img.insNthRoot, img.insBit / 8 + ((img.insBit % 8) ? 1 : 0));
    memcpy(rightNthRoot, img.rightNthRoot, img.rightBit / 8 + ((img.rightBit % 8) ? 1 : 0));
    memcpy(img.curInsNthRoot, img.insNthRoot, img.insBit / 8 + ((img.insBit % 8) ? 1 : 0));
    insBit   = 0;
    insedBit = 0;
    dotLine  = img.dotLine;
    rotation = complex<PRECISION>(cos(par.rotationAngle), sin(par.rotationAngle));
    prevLeft  = -1;
    prevInsed = -1;
    prevIns   = -1;
    prevRight = -1;
    img.spiralBitReversedLeft  = 0;
    img.spiralBitReversedRight = 0;
    img.spiralBitReversedIns   = 0;
    img.spiralBitReversedInsed = 0;
    img.spiralStopReason = 0;
    img.spiralShow  = 0;
    img.spiralTimes = 0;
    while (1) {
        ret = mergeSpiralNthRoot(img.leftBit, insedBit, insBit, img.rightBit,
                                 leftNthRoot, insedNthRoot, insNthRoot, rightNthRoot,
                                 &img.spiralDimension, img.spiralNthRoot);
        if (ret) {
            img.spiralStopReason = 1;
            break;
        }
        if (img.spiralTimes == 0) {
            newtonStart = img.topRoot;
            span = sqrt(norm(img.nextRoot - img.topRoot));
        }
        else if (img.spiralTimes == 1) {
            newtonStart = img.nextRoot;
            prevRoot    = img.topRoot;
            span = sqrt(norm(img.nextRoot - img.topRoot));
        }
        else {
            distance    = img.spiralRoot - prevRoot;
            newtonStart = img.spiralRoot + distance;
            prevRoot    = img.spiralRoot;
            span = SPAN_COEF * sqrt(norm(distance));
            if (img.spiralTimes == 2)  prevDistRatio = INI_D_RATIO;
        }
        img.spiralBitReversed = 0;
        for (mesh = 1; mesh < MESH_MAX; mesh += 2) {
            while (img.threadTempStop)  Sleep(50);
            if (img.threadTerminate)  break;
            if (getRootInBox(newtonStart, span, span, mesh, ALLOWANCE, img.spiralDimension, img.spiralNthRoot, &img.spiralRoot) == true) {
                if (img.spiralTimes > 1) {
                    sDistance = sqrt(norm(img.spiralRoot - prevRoot));
                    dDistance = sqrt(norm(img.spiralRoot - newtonStart));
                    if (sDistance < LDBL_EPSILON * 100)
                        img.spiralStopReason = 2;
                    else
                        prevDistRatio = dDistance / sDistance;
                }
                break;
            }
        }
        if ((mesh == MESH_MAX) && (img.spiralTimes > 1)) {  // trying 1 bit reverse (only 1 bit)
            img.spiralBitReversed = 1;
            for (mesh = 1; mesh < MESH_MAX; mesh += 2) {
                maxRevInsed = MAX_REV_INSED * img.insBit;
                if (insedBit < maxRevInsed)  maxRevInsed = insedBit;
                if (img.rightBit > img.leftBit)  revEnd = img.rightBit;
                else                             revEnd = img.leftBit;
                if (img.insBit > revEnd)         revEnd = img.insBit;
                if (maxRevInsed > revEnd)        revEnd = maxRevInsed;
                img.spiralRetryMeshMax   = MESH_MAX / 2;
                img.spiralRetryBitRevMax = revEnd;
                for (revBit = 0; revBit < revEnd; revBit++) {
                    img.spiralRetryMesh   = mesh / 2 + 1;
                    img.spiralRetryBitRev = revBit + 1;
                    SendMessage(ImageForm->Handle, WM_SPIRAL_RETRY, 0, 0);
                    for (i = 0; i < 4; i++) {
                        switch (i) {
                        case 0:
                            rBit    = img.leftBit;
                            nBit    = img.leftBit;
                            topBit  = img.spiralDimension;
                            prevBit = &prevLeft;
                            pNth    = leftNthRoot;
                            qNth    = img.leftNthRoot;
                            nRev    = &img.spiralBitReversedLeft;
                            break;
                        case 1:
                            rBit    = img.rightBit;
                            nBit    = img.rightBit;
                            topBit  = img.rightBit;
                            prevBit = &prevRight;
                            pNth    = rightNthRoot;
                            qNth    = img.rightNthRoot;
                            nRev    = &img.spiralBitReversedRight;
                            break;
                        case 2:
                            rBit    = insBit;
                            nBit    = insBit;
                            topBit  = img.rightBit + insBit;
                            prevBit = &prevIns;
                            pNth    = insNthRoot;
                            qNth    = img.insNthRoot;
                            nRev    = &img.spiralBitReversedIns;
                            break;
                        case 3:
                            rBit    = maxRevInsed;
                            nBit    = insedBit;
                            topBit  = img.rightBit + insBit + insedBit;
                            prevBit = &prevInsed;
                            pNth    = insedNthRoot;
                            qNth    = 0;
                            nRev    = &img.spiralBitReversedInsed;
                            break;
                        }
                        if (revBit >= rBit)  continue;
                        while (img.threadTempStop)  Sleep(50);
                        if (img.threadTerminate)  break;
                        if (*prevBit < 0)  k = revBit;
                        else {
                            if (revBit == 0)  k = *prevBit;
                            else {
                                if (revBit <= *prevBit)  k = revBit - 1;
                                else                     k = revBit;
                            }
                        }
                        sBit = rightAlign(img.spiralDimension) + 1 - topBit + k;
                        if (isbit(img.spiralNthRoot, sBit) == true)  offOn = 0;
                        else                                         offOn = 1;
                        bitset(img.spiralNthRoot, sBit, offOn);
                        if (getRootInBox(newtonStart, span, span, mesh, ALLOWANCE, img.spiralDimension, img.spiralNthRoot, &img.spiralRoot) == true) {
                            sDistance = sqrt(norm(img.spiralRoot - prevRoot));
                            dDistance = sqrt(norm(img.spiralRoot - newtonStart));
                            if (sDistance < LDBL_EPSILON * 100) {
                                img.spiralStopReason = 2;
                                break;
                            }
                            else {
                                curDistRatio = dDistance / sDistance;
                                if (curDistRatio < DIST_COEF * prevDistRatio) {
                                    prevDistRatio = curDistRatio;
                                    sBit = rightAlign(nBit) + 1 - nBit + k;
                                    bitset(pNth, sBit, offOn);
                                    *prevBit = k;
                                    if (qNth) {
                                        sBit = rightAlign(nBit);
                                        bitcmp(pNth, sBit, qNth, sBit, -nBit, &dum, nRev);
                                    }
                                    else  *nRev += 1;
                                    if (i == 2)  memcpy(img.curInsNthRoot, pNth, img.insBit / 8 + ((img.insBit % 8) ? 1 : 0));
                                    break;
                                }
                            }
                        }
                        if (isbit(img.spiralNthRoot, sBit) == true)  offOn = 0;
                        else                                         offOn = 1;
                        bitset(img.spiralNthRoot, sBit, offOn);
                    }
                    if (i != 4)  break;
                }
                if (revBit < revEnd)  break;
            }
        }
        if (mesh == MESH_MAX) {
            img.spiralStopReason = 2;
            break;
        }
        if (img.threadTerminate) {
            img.spiralStopReason = 3;
            break;
        }
        if (img.spiralStopReason)  break;
        offset  = img.spiralRoot - par.pCenter;
        offset *= rotation;
        img.reverseShowX0 = (offset.real() / par.pSpan[X] + 0.5) * par.imageSize[X];
        img.reverseShowY0 = (- offset.imag() / par.pSpan[Y] + 0.5) * par.imageSize[Y];
        while (img.spiralShow)  { ; }
        SendMessage(ImageForm->Handle, WM_SPIRAL_PAINT, dotLine, 0);
        sleepT = 5000 / (img.spiralTimes + 10);
        if (sleepT < 5)  sleepT = 5;
        Sleep(sleepT);
        while (img.spiralShow != 1)  { ; }
        SendMessage(ImageForm->Handle, WM_SPIRAL_PAINT, dotLine, 0);
        img.spiralTimes++;
        work[0] = 0;
        sBit = rightAlign(insedBit + insBit);
        bitcpy(work, sBit, insNthRoot, rightAlign(insBit), -insBit);
        sBit -= insBit;
        bitcpy(work, sBit, insedNthRoot, rightAlign(insedBit), -insedBit);
        sBit = insedBit + insBit;
        memcpy(insedNthRoot, work, sBit / 8 + ((sBit % 8) ? 1 : 0));
        insedBit += insBit;
        insBit = img.insBit;
        if (demo.onDemo && (img.spiralTimes > demo.spiralTimes)) {
            img.spiralStopReason = 4;
            break;
        }
    }
    SendMessage(ImageForm->Handle, WM_SPIRAL_END, 0, 0);
}
//---------------------------------------------------------------------------
int  mergeSpiralNthRoot(int leftBit, int insedBit, int insBit, int rightBit,
         unsigned char *leftNthRoot, unsigned char *insedNthRoot,
         unsigned char *insNthRoot, unsigned char *rightNthRoot,
         int *dimension, unsigned char *nthRoot)
{
    int lBit, iBit, dBit, rBit, nBit;

    *dimension = leftBit + insedBit + insBit + rightBit;
    if (*dimension > REV_MAX_DIMENSION)  return 1;
    nthRoot[0] = 0;
    lBit = rightAlign(leftBit);
    dBit = rightAlign(insedBit);
    iBit = rightAlign(insBit);
    rBit = rightAlign(rightBit);
    nBit = rightAlign(*dimension);
    bitcpy(nthRoot, nBit, rightNthRoot, rBit, -rightBit);
    nBit -= rightBit;
    bitcpy(nthRoot, nBit, insNthRoot, iBit, -insBit);
    nBit -= insBit;
    bitcpy(nthRoot, nBit, insedNthRoot, dBit, -insedBit);
    nBit -= insedBit;
    bitcpy(nthRoot, nBit, leftNthRoot, lBit, -leftBit);
    return 0;
}
//---------------------------------------------------------------------------
void spiralShow(int dotLine)
{
    char buf[REV_MAX_DIMENSION/4+2], buf1[32];

    if (dotLine == 0) {
        ImageForm->Image->Canvas->Pen->Color = clWhite;
        ImageForm->Image->Canvas->Pen->Style = psSolid;
        ImageForm->Image->Canvas->Pen->Width = 10;
        ImageForm->Image->Canvas->Pen->Mode  = pmXor;
        ImageForm->Image->Canvas->MoveTo(img.reverseShowX0, img.reverseShowY0);
        ImageForm->Image->Canvas->LineTo(img.reverseShowX0, img.reverseShowY0);
    }
    else if (img.spiralShow == 0) {
        if (img.spiralBitReversed == 0)
              ImageForm->Image->Canvas->Pen->Color = clWhite;
        else  ImageForm->Image->Canvas->Pen->Color = clRed;
        ImageForm->Image->Canvas->Pen->Style = psSolid;
        ImageForm->Image->Canvas->Pen->Width = 2;
        ImageForm->Image->Canvas->Pen->Mode  = pmCopy;
        if (img.spiralTimes == 0)  ImageForm->Image->Canvas->MoveTo(img.reverseShowX0, img.reverseShowY0);
        else                       ImageForm->Image->Canvas->LineTo(img.reverseShowX0, img.reverseShowY0);
    }
    if ((img.spiralShow == 0) && (img.threadTerminate == 0)) {
        sprintf(buf, "%d", img.spiralDimension);
        SpiralForm->GDimensionEdit->Text = buf;
        BitsToHexa(img.spiralDimension, img.spiralNthRoot, buf);
        SpiralForm->GNthRootEdit->Text = buf;
        BitsToHexa(img.insBit, img.curInsNthRoot, buf);
        SpiralForm->GInsertEdit->Text = buf;
        sprintf(buf, "%d", img.spiralTimes);
        SpiralForm->GTimesEdit->Text = buf;
        sprintf(buf, "%.17Lf", img.spiralRoot.real());
        SpiralForm->GRootXEdit->Text = buf;
        sprintf(buf, "%.17Lf", img.spiralRoot.imag());
        SpiralForm->GRootYEdit->Text = buf;
        if ((img.spiralBitReversedLeft == 0) && (img.spiralBitReversedIns == 0) &&
            (img.spiralBitReversedInsed == 0) && (img.spiralBitReversedRight == 0)) {
            SpiralForm->ResultMessage("");
        }
        else {
            buf[0] = 0;
            if (img.spiralBitReversedLeft) {
                strcat(buf, "Left ");
                sprintf(buf1, "%d bit reversed. ", img.spiralBitReversedLeft);
                strcat(buf, buf1);
            }
            if (img.spiralBitReversedIns) {
                strcat(buf, "Insert ");
                sprintf(buf1, "%d bit reversed. ", img.spiralBitReversedIns);
                strcat(buf, buf1);
            }
            if (img.spiralBitReversedInsed) {
                strcat(buf, "Inserted ");
                sprintf(buf1, "%d bit reversed. ", img.spiralBitReversedInsed);
                strcat(buf, buf1);
            }
            if (img.spiralBitReversedRight) {
                strcat(buf, "Right ");
                sprintf(buf1, "%d bit reversed. ", img.spiralBitReversedRight);
                strcat(buf, buf1);
            }
            SpiralForm->ResultAlarm(buf);
        }
    }
    img.spiralShow++;
    if (img.spiralShow == 2)  img.spiralShow = 0;
}
//---------------------------------------------------------------------------
void spiralRetry()
{
    char buf[128], buf1[32];

    strcpy(buf, "Trying bit reverse. ");
    sprintf(buf1, "Mesh %d/", img.spiralRetryMesh);
    strcat(buf, buf1);
    sprintf(buf1, "%d , ", img.spiralRetryMeshMax);
    strcat(buf, buf1);
    sprintf(buf1, "Bit %d/", img.spiralRetryBitRev);
    strcat(buf, buf1);
    sprintf(buf1, "%d", img.spiralRetryBitRevMax);
    strcat(buf, buf1);
    SpiralForm->ResultAlarm(buf);
}
//---------------------------------------------------------------------------
void spiralStop()
{
    if (img.onDrawing != Non)  TerminateThread((void *)img.threads->Handle, 0);
    calculationStopped();
}
//---------------------------------------------------------------------------
void spiralEndMessage()
{
    ControlForm->StartButton->Caption = "Start";
    ImageForm->enableCyclingAuto();
    switch (img.spiralStopReason) {
    case 1:
        SpiralForm->ResultAlarm("Dimension over. (16384)");
        break;
    case 2:
        SpiralForm->ResultAlarm("Root not found.");
        break;
    case 3:
        SpiralForm->ResultMessage("Stopped.");
        break;
    }
    SpiralForm->GoButton->Caption = "Go";
}
//---------------------------------------------------------------------------
void changeToBoxMode()
{
    img.savedZoomMode = img.zoomMode;
    img.zoomMode = 0;
    ZoomForm->ShowZoomModeChecked();
}
//---------------------------------------------------------------------------
void restoreZoomMode()
{
    img.zoomMode = img.savedZoomMode;
    ZoomForm->ShowZoomModeChecked();
}
//---------------------------------------------------------------------------
void startLow()
{
    int mask[] = { 0xffffffff, 0xffffffff, 0xfffffffe, 0xfffffffc,
                   0xfffffff8, 0xfffffff0, 0xffffffe0, 0xffffffc0,
                   0xffffff80, 0xffffff00, 0xfffffe00, 0xfffffc00,
                   0xfffff800, 0xfffff000, 0xffffe000, 0xffffc000,
                   0xffff8000, 0xffff0000
                 };

    low.nthRoot++;
    if (mask[low.dimension + 1] & low.nthRoot) {
        low.dimension++;
        low.nthRoot = 0;
    }
    if (low.dimension > low.toDimension) {
        low.onDemo = 0;
        ImageForm->changeStaticMenu();
        if (!demo.onDemo) {
            img.maxIterAuto = 0;
            img.onDrawing = Non;
            ImageForm->changeStaticMenu();
            ImageForm->Caption = "Mandelbrot Root Explorer (Classification and Auto View)";
            ControlForm->StartButton->Caption = "Start";
            ControlForm->ShowAll();
        }
        if (demo.onDemo)  startDemo();
        return;
    }
    par.dimension = low.dimension;
    if (low.dimension <= 8) {
        par.nthRoot[0] = (unsigned char)low.nthRoot;
    }
    else {
        par.nthRoot[0] = (unsigned char)(low.nthRoot >> 8);
        par.nthRoot[1] = (unsigned char)low.nthRoot;
    }
    par.nthRootAtRandom = 0;
    par.periodicBitsNumber = 1;
    par.topBitsNumber = 0;
    par.lastBitsNumber = 0;
    par.maxIter = 128 + 16 * low.dimension;
    par.rotationAngle = 0.0;
    RootChangedNonStop();
    img.maxIterAuto = 0;
    ImageForm->changeSaveMenu();
    if (img.rootFindNonStop)  FindingRootForm->NonStopCheckBox->Checked = false;
    ControlForm->ShowAll();
    if (demo.onDemo) {
        strcpy(demo.caption, "Mandelbrot Root Explorer : Low Dimension Demo. Dimension = ");
        strcat(demo.caption, ControlForm->DimensionEdit->Text.c_str());
        strcat(demo.caption, ", Nth Root = ");
        strcat(demo.caption, ControlForm->NthRootEdit->Text.c_str());
    }
    rootImageCalculationStart();
}
//---------------------------------------------------------------------------

